/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import net.java.javafx.ui.JBookPanel;
import net.java.javafx.ui.JBookPanel$BookPanelViewModel as BookPanelViewModel;
import java.awt.Rectangle;
import javax.swing.event.ChangeListener;

public class BookPanel extends Widget {
    private attribute bookPanel: JBookPanel;
    public attribute pages: Widget*;
    public attribute leftPageIndex: Integer;
    public attribute pageBounds: Rectangle?;
    public attribute refreshSpeed: Number;
    public attribute darkShadowColor: Color?;
    public attribute neutralShadowColor: Color?;
    public attribute shadowWidth: Number;
    public attribute softClipping: Boolean;
    public attribute borderLinesVisible: Boolean;
    public operation nextPage();
    public operation previousPage();
}

attribute BookPanel.pageBounds = {x: 70, y: 80, width: 210, height: 342};
attribute BookPanel.background = white:Color;
attribute BookPanel.refreshSpeed = 20;
attribute BookPanel.borderLinesVisible = false;
attribute BookPanel.focusable = false;

trigger on BookPanel.leftPageIndex = value {
    bookPanel.setLeftPageIndex(value);
}

function BookPanel.nextPage() = bookPanel.nextPage();
function BookPanel.previousPage() = bookPanel.previousPage();


operation BookPanel.createComponent() {
    var self = this;
    bookPanel = new JBookPanel();
    bookPanel.setOpaque(false);
    bookPanel.setModel(new BookPanelViewModel() {
            operation getPageCount() {
                return sizeof self.pages;
            }
            operation getPage(i:Number) {
                return self.pages[i].getComponent();
            }
        });
    bookPanel.addSelectionListener(new ChangeListener() {
            operation stateChanged(e) {
                self.leftPageIndex = self.bookPanel.getLeftPageIndex();
            }
        });
    if (pageBounds <> null) {
        bookPanel.setMargins(pageBounds.x, pageBounds.y, pageBounds.width, pageBounds.height);
    }
    bookPanel.setRefreshSpeed(refreshSpeed);
    bookPanel.setBorderLinesVisible(borderLinesVisible);
    bookPanel.setLeftPageIndex(leftPageIndex);
    return bookPanel;
}
