/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.Keyboard;
import javafx.ui.KeyStroke;
import javafx.ui.canvas.*;
import java.awt.MouseInfo;
import javax.swing.JComponent;
import net.java.javafx.ui.UIContext;
import net.java.javafx.jazz.util.ZCanvas;
import net.java.javafx.jazz.ZTransformGroup;
import net.java.javafx.jazz.ZFilterGroup;
import net.java.javafx.jazz.ZSceneGraphObject;
import net.java.javafx.jazz.ZNode;
import net.java.javafx.jazz.ZVisualComponent;
import net.java.javafx.jazz.ZDrawingSurface;
import net.java.javafx.jazz.event.ZNodeListener;
import net.java.javafx.jazz.event.ZNodeEvent;
import net.java.javafx.jazz.event.ZMouseListener;
import net.java.javafx.jazz.event.ZMouseMotionListener;
import net.java.javafx.jazz.event.ZMouseEvent;
import net.java.javafx.jazz.util.ZBounds;
import javax.swing.ImageIcon;
import java.awt.Dimension;
import java.awt.event.KeyListener;
import java.awt.event.ComponentListener;
import java.lang.Math;
import net.java.javafx.ui.F3MouseListener;
import net.java.javafx.ui.F3NodeListener;
import java.lang.System;
import javax.swing.JViewport;
import org.jdesktop.layout.Baseline;
import net.java.javafx.jazz.util.ZCanvas$ComputeBaseline as ComputeBaseline;

public class Canvas extends Widget, CanvasElement, Container {
    // private:
    protected attribute scaleToFitList: Node*;
    protected attribute sizeToFitList: SizeableCanvasElement*;
    protected attribute zcanvas: ZCanvas;
    protected  attribute root: ZTransformGroup;
    public attribute logRepaint: Boolean;
    public attribute logPrefix: String;
   
    protected attribute focusRect: Rect?;
    protected attribute focusBounds: ZBounds?;
    protected attribute focusedNode: Node?;   
    protected operation getF3(obj:ZSceneGraphObject):Node;  

    // public:
    public attribute content: Node*;
    public attribute scaleToFit: Boolean;
    public attribute viewport: CanvasViewport;
    public operation pick(x: Number, y: Number, w:Number, h:Number): Node*;
    public operation print();

    /**
     * Optional handler called when the user drops an object into this canvas.
     */
    public attribute onDrop: function(e:CanvasDropEvent)?;
    public attribute onDragEnter: function(e:CanvasDropEvent)?;
    public attribute onDragExit: function(e:CanvasDropEvent)?;
    /**
     * Optional filter for the types of objects that may be dropped into this
     * canvas.
     */
    public attribute dropType: Class?;
    /**
     * <code>attribute acceptDrop: function(e:CanvasDropEvent): Boolean</code><br></br>
     * Optional handler called when the user drops an object into this canvas.
     * If it returns false, the drop is rejected. 
     */
    public attribute canAcceptDrop: function(e:CanvasDropEvent):Boolean?;

    private operation acceptDrop(value):Boolean;
    private operation handleDragEnter(value);
    private operation handleDragExit(value);
    private operation getDragNode(): Node;
    private operation getDragValue(): Node;
    private operation setDropValue(value);
    operation doExportAsDrag(e:CanvasDragEvent);
    attribute dragNode: Node;
    attribute dragValue: *;
    attribute dropTargetNode: Node?;
}
attribute Canvas.border = select EmptyBorder {top: n, left: n, right: n, bottom: n} from n in 5;
attribute Canvas.focusBounds = bind focusedNode.bounds;

trigger on Canvas.focusBounds = value {
if (false) {
    if (value <> null) {
        if (focusRect == null) {
            focusRect = Rect {stroke: black, strokeDashArray: [2] strokeWidth: 0.5};
            focusRect.parentCanvasElement = this;
            zcanvas.getLayer().addChild(focusRect.getNode());
        }
        var b = focusedNode.getGlobalBounds();
        focusRect.x = b.x;
        focusRect.y = b.y;
        focusRect.width = b.width;
        focusRect.height = b.height;
        focusRect.visible = true;
    } else {
        focusRect.visible = false;
    }
}
}


trigger on Canvas.focusedNode = newValue {
	println("focused node = {newValue}");
}

trigger on Canvas.logRepaint = value {
    zcanvas.setLogRepaint(value);
}

trigger on Canvas.logPrefix = value {
    zcanvas.setLogPrefix(value);
}

operation Canvas.getF3(obj:ZSceneGraphObject) {
    while (obj <> null) {
        var n = (Node)obj.getClientProperty("f3");
        if (n <> null) {
            return n;
        }
        if (obj instanceof ZNode) {
            obj = ((ZNode)obj).getParent();
        } else if (obj instanceof ZVisualComponent) {
            obj = ((ZVisualComponent)obj).getParents()[0];
        } else {
            obj = null;
        }
    }
    return null;
}

operation Canvas.pick(x: Number, y: Number, w: Number, h: Number) {
    var result:Node* = [];
    var path = zcanvas.getDrawingSurface().pick(x, y, w, h);
    var n = getF3(path.getObject());
    if (n <> null) {
        insert n into result;
    }
    var lastNode = n;
    for (i in reverse [0..<path.numParents]) {
        var p = path.getParent(i);
        var n = getF3(p);
        if (n <> null) {
            if (n <> lastNode) {
                insert n into result;
            }
            lastNode = n;
        }
    }
    return result;
}

operation Canvas.print() {
	zcanvas.getDrawingSurface().printSurface();
}

attribute Canvas.viewport = CanvasViewport {};

// Canvas

attribute Canvas.zcanvas = new ZCanvas;

operation Canvas.getCanvas() {
    return this;
}

trigger on insert c into Canvas.content {
    c.parentCanvasElement = this;
    if (root <> null) {
        root.addChild(c.getNode());
    }
}

trigger on delete c from Canvas.content {
    if (root <> null) {
        root.removeChild(c.getNode());
    }
}

trigger on Canvas.content[oldValue] = newValue {
    newValue.parentCanvasElement = this;
    if (root <> null) {
        root.removeChild(oldValue.getNode());
        root.addChild(newValue.getNode());
    }
}

operation Canvas.createComponent() {
    root = new ZTransformGroup();
    zcanvas.setCursor(DEFAULT:Cursor.getCursor());
    zcanvas.setComputeBaseline(new ComputeBaseline() {
	    operation getBaseline(jcomponent:JComponent, x:Number, y:Number) {
		return Baseline.getBaseline(jcomponent);
	    }
	});
    zcanvas.setOpaque(false);
    zcanvas.getLayer().addChild(root);
    for (i in content) {
        i.parentCanvasElement = this;
        root.addChild(i.getNode());
    }
    zcanvas.setNavEventHandlersActive(false);
    zcanvas.setSwingEventHandlersActive(true);
    zcanvas.setEnableNodeEvents(true);
    var self = this;
    var hoverListener = new F3MouseListener();
    zcanvas.addZMouseListener(hoverListener);
    zcanvas.addZMouseMotionListener(hoverListener);
    zcanvas.addComponentListener(new ComponentListener() {
	    operation componentShown(e) {
                var canvasBounds = self.zcanvas.getBounds();
                var parent = self.zcanvas.getParent();
                if (parent instanceof JViewport) {
                    canvasBounds = parent.getBounds();
                }
                self.viewport.setSize(canvasBounds.width, canvasBounds.height);

		if (sizeof self.scaleToFitList > 0) {
                    for (n in self.scaleToFitList) {
                        var root = (ZTransformGroup)n.getNode();
                        var nodeBounds = root.getBounds();
                        if ((nodeBounds.width == 0) or (nodeBounds.height == 0)) {
                            return;
                        }
                        var t = root.getTransform();
                        t.scale((canvasBounds.width -nodeBounds.x) / nodeBounds.width,
                                (canvasBounds.height -nodeBounds.y) /nodeBounds.height);
                        root.setTransform(t);
                    
                    }
                }
		if (sizeof self.sizeToFitList > 0) {
                    for (n in self.sizeToFitList) {
                        n.setSize(canvasBounds.width.intValue(), canvasBounds.height.intValue());
                    }
                }
            }
	    operation componentResized(e) {
                var parent = self.zcanvas.getParent();
                var canvasBounds = self.zcanvas.getBounds();
                if (parent instanceof JViewport) {
                    canvasBounds = parent.getBounds();
                }
                self.viewport.setSize(canvasBounds.width, canvasBounds.height);
		if (sizeof self.scaleToFitList > 0) {
                    for (n in self.scaleToFitList) {
                        var root = (ZTransformGroup)n.getNode();
                        var nodeBounds = root.getBounds();
                        if ((nodeBounds.width == 0) or (nodeBounds.height == 0)) {
                            // no action
                        } else {
                            var t = root.getTransform();
                            t.scale((canvasBounds.width -nodeBounds.x) / nodeBounds.width,
                                    (canvasBounds.height -nodeBounds.y) /nodeBounds.height);
                            root.setTransform(t);
                        }
                    }
                }
		if (sizeof self.sizeToFitList > 0) {
                    for (n in self.sizeToFitList) {
                        n.setSize(canvasBounds.width.intValue(), canvasBounds.height.intValue());
                    }
                }
	    }
	});
    zcanvas.getDrawingSurface().setRenderQuality(ZDrawingSurface.RENDER_QUALITY_HIGH);
    zcanvas.getDrawingSurface().setInteractingRenderQuality(ZDrawingSurface.RENDER_QUALITY_HIGH);
    <<javax.swing.ToolTipManager>>.sharedInstance().registerComponent(zcanvas);
    var javaVersion = (String)System.getProperty("java.version");
    var javaBroken = javaVersion in (select "1.5.0_0{i}" from i in [1..9]);
    context:UIContext.addTransferHandler(zcanvas, dropType,
                                         new <<net.java.javafx.ui.ValueGetter>>() {
                                             operation get() {
                                                 return self.getDragValue();
                                             }
                                         },
                                         new <<net.java.javafx.ui.ValueSetter>>() {
                                             operation set(value) { println("set {value}");
                                                 self.setDropValue(value);
                                                 
                                             }
                                         },
                                         new <<net.java.javafx.ui.ValueAcceptor>>() {
                                             operation accept(value) { println("canvas accept {value}");
                                                 if (javaBroken) {
                                                     return true;
                                                 }
                                                 var result = self.acceptDrop(value);
                                                 return result;
                                             }
                                             operation dragEnter(value) {
 println("dragEntr {value}");
                                                 self.handleDragEnter(value);
                                             }
                                             operation dragExit(value) {
 println("dragExit {value}");
                                                 self.handleDragExit(value);
                                             }
                                         },
                                         new <<net.java.javafx.ui.VisualRepresentation>>() {
                                             operation getIcon(value) {
                                                 var c = CanvasIcon {
                                                     content: self.getDragNode()
                                                 };
                                                 var icon = c.getIcon();
                                                 return icon;
                                             }
                                         });
     zcanvas.addKeyListener(new KeyListener() {
                operation keyTyped(e:AWTKeyEvent) {
                    (self.focusedNode.onKeyTyped)(self.makeKeyEvent(e));
                }
                operation keyPressed(e:AWTKeyEvent) {
                    (self.focusedNode.onKeyDown)(self.makeKeyEvent(e));
                }
                operation keyReleased(e:AWTKeyEvent) {
                    (self.focusedNode.onKeyUp)(self.makeKeyEvent(e));
                }
     });
     return zcanvas;
}

operation Canvas.raiseNode(n:Node) {
    var i = select indexof x from x in content where x == n;
    if (i == sizeof content -1) {
        return;
    }
    delete content[i];
    insert n after content[i];
}
operation Canvas.lowerNode(n:Node) {
    var i = select first indexof x from x in content where x == n;
    if (i == 0) {
        return;
    }
    delete content[i];
    insert n before content[i-1];
}

operation Canvas.moveNodeToFront(n:Node) {
    var i = select indexof x from x in content where x == n;
    delete content[i];
    insert n as last into content;
}

operation Canvas.moveNodeToBack(n:Node) {
    var i = select indexof x from x in content where x == n;
    delete content[i];
    insert n as first into content;
}

operation Canvas.setDropValue(value) {
    println("CANVAS set drop value: {value}");
    var info = MouseInfo.getPointerInfo();
    var p = info.getLocation();
    var location = zcanvas.getLocationOnScreen();
    p.x -= location.x;
    p.y -= location.y;
    var path = pick(p.x, p.y, 1, 1);
    var e = CanvasDropEvent {
        x: p.x,
        y: p.y
        transferData: value
    };
    for (i in path) {
        if (i.handleDrop(e)) {
            return;
        }
    }
    dropTargetNode.handleDragExit(e);
    dropTargetNode = null;
    e.localX = e.x;
    e.localY = e.y;
    (this.onDrop)(e);
}

operation Canvas.acceptDrop(value) {
    println("CANVAS accept drop: {value}");
    var info = MouseInfo.getPointerInfo();
    var p = info.getLocation();
    var location = zcanvas.getLocationOnScreen();
    p.x -= location.x;
    p.y -= location.y;
    var path = pick(p.x, p.y, 1, 1);
    var e = CanvasDropEvent {
        x: p.x,
        y: p.y
        transferData: value
    };
    for (i in path) {
        if (i.handleAcceptDrop(e)) {          
           if (dropTargetNode <> i) {
               dropTargetNode.handleDragExit(e);
           }
           dropTargetNode = i;
           return true;
        }
    }
    dropTargetNode.handleDragExit(e);
    dropTargetNode = null;
    if (this.canAcceptDrop <> null) {
        e.localX = e.x;
        e.localY = e.y;
        return (this.canAcceptDrop)(e);
    }
    return onDrop <> null;
}

operation Canvas.getDragNode() {
    return dragNode;
}

operation Canvas.getDragValue() {
    return dragValue;
}

operation Canvas.handleDragEnter(value) {
    println("drag enter {value}");
    var info = MouseInfo.getPointerInfo();
    var p = info.getLocation();
    var location = zcanvas.getLocationOnScreen();
    p.x -= location.x;
    p.y -= location.y;
    var path = pick(p.x, p.y, 1, 1);
    var e = CanvasDropEvent {
        x: p.x,
        y: p.y
        transferData: value
    };
    for (i in path) {
        if (i.handleDragEnter(e)) {
           if (dropTargetNode <> i) {
               dropTargetNode.handleDragExit(e);
           }
           dropTargetNode = i;
           break;
        }
    }
    if (this.onDragEnter <> null) {
        e.localX = e.x;
        e.localY = e.y;
        (this.onDragEnter)(e);
    }
}

operation Canvas.handleDragExit(value) {
    println("drag exit {value}");
    var info = MouseInfo.getPointerInfo();
    var p = info.getLocation();
    var location = zcanvas.getLocationOnScreen();
    p.x -= location.x;
    p.y -= location.y;
    var path = pick(p.x, p.y, 1, 1);
    var e = CanvasDropEvent {
        x: p.x,
        y: p.y
        transferData: value
    };
    for (i in path) {
        if (i.handleDragExit(e)) {
           break;
        }
    }
    if (this.onDragExit <> null) {
        e.localX = e.x;
        e.localY = e.y;
        (this.onDragExit)(e);
    }
}

operation Canvas.doExportAsDrag(e:CanvasDragEvent) {
     //dragNode = e.source;
     dragNode = e.dragView;
     dragValue = e.dragValue;
}
