/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.ComboBoxCell;
import javafx.ui.SingleSelection;
import java.awt.event.ActionListener;
import javax.swing.JTextField;
import javax.swing.ComboBoxEditor;

public class ComboBox extends Widget {
    private attribute listeners:<<javax.swing.event.ListDataListener>>*;
    private attribute jcombobox:<<javax.swing.JComboBox>>;

    public attribute cells: ComboBoxCell*;
    public attribute selection: Number;
    public attribute editable: Boolean?;
    public attribute lightweightPopupEnabled: Boolean?;
    public attribute maximumRowCount: Number?;
    public attribute action: function()?;
    public attribute value: String?; 
}

attribute ComboBox.cells = {text: " "};
attribute ComboBox.cursor = DEFAULT;

trigger on (new ComboBox) {
    selection = -1;
    jcombobox = new <<net.java.javafx.ui.F3ComboBox>>();
    //jcombobox.setLightWeightPopupEnabled(false);
    jcombobox.setSelectedIndex(-1);
    var self = this;
    jcombobox.addActionListener(new ActionListener() {
             operation actionPerformed(e) {
                if (self.jcombobox.getSelectedIndex() >= 0) {
                   (self.action)();
                }
             }
        });
    var tf = TextField {
        value: bind value
    };
    var jt = (JTextField)tf.getComponent();
    jcombobox.setEditor(new ComboBoxEditor() {
            operation setItem(anItem) {
                var c = (ComboBoxCell)anItem;
                var value = c.value;
                jt.setText("{value}");
            }
            operation getItem() {
                var str = jt.getText();
                return self.cells[self.selection];
            }
            operation selectAll() {
                jt.selectAll();
                jt.requestFocus();
            }
            operation getEditorComponent() {
                return tf.getComponent();
            }
            operation addActionListener(l:ActionListener) {
                //jt.addActionListener(l);
            }
            operation removeActionListener(l:ActionListener) {
                //jt.removeActionListener(l);
            }
        });
    jcombobox.setModel(new <<javax.swing.ComboBoxModel>>() {
                           operation getSize() {
                               return sizeof self.cells;
                           }
                           operation getElementAt(i:Number) {
                               return self.cells[i];
                           }
                           operation addListDataListener(l:<<javax.swing.event.ListDataListener>>) {
                               insert l into self.listeners;
                           }
                           operation removeListDataListener(l:<<javax.swing.event.ListDataListener>>) {
                               delete self.listeners[x|x == l];
                           }
                           operation setSelectedItem(anItem) {
                               var i = select indexof x from x in self.cells where x == (ComboBoxCell) anItem;
                               if (i == null) {
                                   i = -1;
                               }
                               self.selection = i[0];
                               jt.setText("{self.cells[self.selection].value}");
                           }
                           operation getSelectedItem() {
                               return self.cells[self.selection];
                           }
                       });
    jcombobox.addItemListener(new <<java.awt.event.ItemListener>>() {
                                  operation itemStateChanged(e) {
                                      var i = self.jcombobox.getSelectedIndex();
                                      if (i <> null) {
                                          self.selection = i;
                                      }
                                  }
                              });
    jcombobox.setRenderer(new <<javax.swing.ListCellRenderer>>() {
                              operation getListCellRendererComponent(jlist:<<javax.swing.JList>>, obj,
                                                                    i:Number, selected:Boolean, focused:Boolean) {
                                 var cell = (ComboBoxCell)obj;
                                 return context:<<net.java.javafx.ui.UIContext>>.getListCellRendererComponent(jlist,
                                                                                                      cell.text,
                                                                                                      i, selected, focused,
                                                                                                      cell.toolTipText);
                             }
                             operation equals(o) {
                                 return o == this;
                             }
                          });
}

operation ComboBox.createComponent() {
    return jcombobox;
}


trigger on (insert cell into ComboBox.cells) {
    cell.combobox = this;
    var sel = selection;
    var e:<<javax.swing.event.ListDataEvent>>;
    e = new <<javax.swing.event.ListDataEvent>>(this, e.INTERVAL_ADDED,
                                                indexof cell, indexof cell);
    for (i in listeners) {
        i.intervalAdded(e);
    }
    if (indexof cell == sel) {
        jcombobox.setSelectedIndex(indexof cell);
    }
}

trigger on (delete cell from ComboBox.cells) {
    var e:<<javax.swing.event.ListDataEvent>>;
    e = new <<javax.swing.event.ListDataEvent>>(this, e.INTERVAL_REMOVED,
                                                indexof cell, indexof cell);
    for (i in listeners) {
        i.intervalRemoved(e);
    }
}

trigger on (ComboBox.editable = value) {
    if (value <> null) {
        jcombobox.setEditable(value);
    }
}

trigger on (ComboBox.lightweightPopupEnabled = value) {
    if (value <> null) {
        jcombobox.setLightWeightPopupEnabled(value);
    }
}

trigger on (ComboBox.maximumRowCount = value) {
    if (value <> null) {
        jcombobox.setMaximumRowCount(value);
    }
}

trigger on (ComboBox.selection[old] = value) {
    if (value >= 0 and value < sizeof cells) {
        jcombobox.setSelectedIndex(value);
        jcombobox.repaint();
    }
}

