/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.AbstractFrame;
import javafx.ui.UIElement;
import javafx.ui.MenuBar;
import javafx.ui.Widget;
import javafx.ui.Dialog;
import javafx.ui.Image;
import java.awt.*;
import java.awt.Dimension;
import net.java.javafx.ui.UIContext;
import javafx.ui.canvas.Shape;
/*
import com.sun.jna.examples.ShapedWindow;
import com.l2fprod.gui.nativeskin.NativeSkin;
*/
/**
 * A <code>Frame</code> is a top-level window with a title and a border,
 * and an optional menu bar.
 */
public class Frame extends AbstractFrame {
        
    private attribute winListener: <<java.awt.event.WindowListener>>;
    private attribute compListener: <<java.awt.event.ComponentListener>>;
    private attribute frame: <<javax.swing.JFrame>>;
    private attribute inListener: Boolean;
    public attribute shape: Shape?;
    public attribute disposeOnClose: Boolean;
    public attribute hideOnClose: Boolean;
    public attribute owner: UIElement?;
    public attribute screenx: Number?;
    public attribute screeny: Number?;
    public attribute menubar: MenuBar?;
    public attribute content: Widget;
    public attribute dispose: Boolean;
    /**
     * This is the title of the frame.  It can be changed
     * at any time.  
     */
    public attribute title: String;
    public attribute height: Number?;
    public attribute width: Number?;
    public operation show();
    public operation toFront();
    public operation toBack();
    public operation hide();
    public operation showDialog(d:Dialog);
    public attribute onOpen: function();
    public attribute onClose: function();
    public attribute centerOnScreen: Boolean;
    public attribute background: AbstractColor?;
    /**
     * Makes the frame visible or invisible. Frame's are initially invisible.
     * You must explicitly assign <code>true</code> to this attribute to make the frame
     * visible on the screen.
     */
    public attribute visible: Boolean;
    /**
     * This field indicates whether the frame is resizable.
     * This property can be changed at any time.
     * <code>resizable</code> will be true if the frame is
     * resizable, otherwise it will be false.
     */
    public attribute resizable: Boolean;
    /**
     * The icon image for this frame, or <code>null</code> 
     * if this frame doesn't have an icon image.
     */
    public attribute iconImage: Image?;
    /**
     * Disables or enables decorations for this frame.
     * This attribute can only be set while the frame is not displayable.
     */
    public attribute undecorated: Boolean;
    public attribute showing: Boolean;
    public attribute iconified: Boolean;
    public attribute active: Boolean;
    public operation move(dx: Number, dy: Number);
    public operation setLocation(x: Number, y: Number);
    public operation resize(dx: Number, dy: Number);
    public operation setSize(width: Number, height: Number);
    protected operation setContentPane(widget:Widget);
    public operation pack();
    public operation close();
}

attribute Frame.iconified = false;
attribute Frame.hideOnClose = true;

trigger on Frame.background = value {
   frame.setBackground(value.getColor());
}

trigger on Frame.active = value {
    if (inListener == false) {
        if (value == true) {
            toFront();
        } else {
            toBack();
        }
    }
}

trigger on Frame.iconified = value {
    var state = frame.getExtendedState();
    var newState = 0;
    if (value) {
        newState = context:UIContext.setBit(state, frame.ICONIFIED);
    } else {
        newState = context:UIContext.clearBit(state, frame.ICONIFIED);
    }
    if (state <> newState) {
        frame.setExtendedState(newState);
    }
}

attribute Frame.resizable = true;
attribute Frame.disposeOnClose = true;

operation Frame.move(dx: Number, dy: Number) {
    var loc = frame.getLocation();
    frame.setLocation(loc.x + dx, loc.y + dy);
    inListener = true;
        screenx += dx;
        screeny += dy;
    inListener = false;
}

operation Frame.setLocation(x: Number, y: Number) {
    frame.setLocation(x, y);
    inListener = true;
        screenx = x;
        screeny = y;
    inListener = false;
}

operation Frame.resize(dx: Number, dy: Number) {
    var size = frame.getSize();
    frame.setSize(size.width + dx.intValue(), size.height + dy.intValue());
    inListener = true;
        width += dx.intValue();
        height += dy.intValue();
    inListener = false;
}

operation Frame.setSize(w: Number, h: Number) {
    inListener = true;
    frame.setSize(w, h);
        width = w;
        height = h;
    inListener = false;
}

trigger on Frame.screenx = n {
    if (inListener) {
        return;
    }
    frame.setLocation(new <<java.awt.Point>>(screenx, screeny));
}

trigger on Frame.screeny = n {
    if (inListener) {
        return;
    }
    frame.setLocation(new <<java.awt.Point>>(screenx, screeny));
}

trigger on (Frame.iconImage = value) {
    frame.setIconImage(value.getImage());
}

trigger on (Frame.resizable = value) {
    frame.setResizable(value);
}

trigger on (Frame.undecorated = value) {
    frame.setUndecorated(value);
}

trigger on (Frame.visible = value) {
    if (inListener) {
        return;
    }
    if (value) {
        do later {
            this.show();
        }
    } else {
        if (disposeOnClose) {
            disposeOnClose = false;
            this.close();
        } else {
            frame.visible = false;
        }
    }
}
trigger on (new Frame) {
    if (not context:UIContext.isActive()) {
        return;
    }
    frame = new <<javax.swing.JFrame>>;
    if (background <> null) {
        frame.setBackground(background.getColor());
    }
    frame.getRootPane().putClientProperty("apple.awt.draggableWindowBackground", false);
    frame.setDefaultCloseOperation(frame.DO_NOTHING_ON_CLOSE);
    //frame.getRootPane().getGlassPane().setVisible(true);
    context:UIContext.registerWindow(frame);
    win = frame;
    var self = this;
    winListener = new <<java.awt.event.WindowListener>>() {
                              operation windowClosing(e) {
                                  if (self.disposeOnClose) {
                                       self.frame.removeWindowListener(self.winListener);
                                       self.frame.removeComponentListener(self.compListener);
                                       context:UIContext.unregisterWindow(self.frame);
                                       self.frame.dispose(); // fix me...
                                  }
                                  (self.onClose)();
                                  if (self.hideOnClose) {
                                      self.visible = false;
                                      self.showing = false;
                                  }
                              }
                              operation windowOpened(e) {
                                      (self.onOpen)();
                                }
                              operation windowIconified(e) {
                                      self.iconified = true;
                              }
                              operation windowDeiconified(e) {
                                      self.iconified = false;
                              }
                              operation windowActivated(e) {
                                      self.inListener = true;
                                      self.active = true;
                                      self.inListener = false;
                              }
                              operation windowDeactivated(e) {
                                      self.inListener = true;
                                      self.active = false;
                                      self.inListener = false;
                              }
                          };
    compListener = new <<java.awt.event.ComponentListener>>() {
                operation componentResized(e) {
                    if (self.resizable) {
                        self.inListener = true;
                        self.height = self.frame.getHeight();
                        self.width = self.frame.getWidth();
                        self.inListener = false;
                    }
                }
                operation componentShown(e) {
                    self.inListener = true;
                    self.visible = true;
                    self.inListener = false;
                    self.showing = true;
                }
                operation componentHidden(e) {
                    self.inListener = true;
                    self.visible = false;
                    self.inListener = false;
                    self.showing = false;
                }
                operation componentMoved(e) {
                    if (true) {
                        self.inListener = true;
                        var pt = self.frame.getLocation();
                        self.screenx = pt.getX();
                        self.screeny = pt.getY();
                        self.inListener = false;
                    }
                }
        };
   win.addWindowListener(winListener);
   win.addComponentListener(compListener);
}

trigger on Frame.shape = value {
}

trigger on (Frame.menubar = value) {
    frame.setJMenuBar(value.jmenubar);
}

trigger on (Frame.title = value) {
    frame.setTitle(value);
}

trigger on (Frame.content = value) {
    this.setContentPane(value);
}

operation Frame.setContentPane(w:Widget) {
    if (w <> null) {
        frame.setContentPane(w.getComponent());
        frame.validate();
    }
}

operation Frame.pack() {
    frame.pack();
}


trigger on (Frame.height = value) {
    if (inListener) {
        return;
    }
    if (value <> null) {
        var dim = frame.getSize();
        dim.height = value;
        frame.setSize(dim);
    }
}

trigger on (Frame.width = value) {
    if (inListener) {
        return;
    }
    if (value <> null) {
        var dim = frame.getSize();
        dim.width = value;
        frame.setSize(dim);
    }
}

operation Frame.show() {
    if (not visible) {
        return;
    }
    if (height == null or width == null) {
        this.pack();
        var dim = frame.getSize();
        if (height <> null) {
            dim.height = height;
        }
        if (width <> null) {
            dim.width = width;
        }
        if (height <> null or width <> null) {
            frame.setSize(dim);
        }
    } else {
        frame.pack();
        frame.setSize(new Dimension(width, height));
    }
    if (owner <> null) {
        frame.setLocationRelativeTo(owner.getWindow());
    } else {
        if (screenx <> null and screeny <> null) {
            frame.setLocation(screenx, screeny);
        } else if (centerOnScreen) {
            var d = <<java.awt.Toolkit>>.getDefaultToolkit().getScreenSize();                 
            var s = frame.getSize();
            frame.setLocation(d.width/2 - s.width/2, d.height/2 - s.height/2);
        }
    }
    do later {
        if (not visible) {
            return;
        }
        if (not context:UIContext.isActive()) {
           disposeOnClose = false;
           frame.dispose();
           return;
        }
        /*
        shape.getNode();
        var bg = background.getColor();
        if (bg <> null) {
            var alpha = bg.getAlpha();
            //println("alpha={alpha}");
            if (alpha <> 255) {
                //println("setting transparency to {alpha}");
                NativeSkin.getInstance().setWindowTransparency(frame, alpha);
            }
        }
        var mask = shape.getTransformedShape();
        if (mask <> null) {
            //println("setting mask to {mask}");
            ShapedWindow.setWindowMask(frame, mask);
        }
        */
        frame.setVisible(true);
        frame.toFront();
        var loc = frame.getLocation();
        inListener = true;
        screenx = loc.getX();
        screeny = loc.getY();
        var size = frame.getSize();
        height = size.height;
        width = size.width;
        inListener = false;
    }
}

operation Frame.hide() {
    frame.visible = false;
    showing = false;
}

trigger on Frame.dispose = value {
   if (value) {
        this.close();
   }
}

operation Frame.close() {
    context:UIContext.unregisterWindow(frame);
    disposeOnClose = false;
    frame.dispose();
    showing = false;
    frame = null;
}

operation Frame.toFront() {
    do later {
       frame.toFront();
    }
}

operation Frame.toBack() {
    do later {
        frame.toBack();
    }
}

