/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.GroupLayout;

/**
 * Container which provides Group layout. You layout its content by creating
 * row and column definitions, and then assigning the contained elements
 * <code>row</code> and <code>column</code> attributes.
 */

public class GroupPanel extends Widget, GroupLayout {
    private attribute panel:<<javax.swing.JPanel>>;
    private attribute layout:<<org.jdesktop.layout.GroupLayout>>;

    /** 
     * Determines whether to automatically create gaps between elements
     * based on platform specific UI guidelines. Defaults to true.
     */
    public attribute autoCreateGaps: Boolean;
    /** 
     * Determines whether to automatically create container gaps between 
     * contained elements and this object based on platform specific UI 
     * guidelines. Defaults to true.
     */
    public attribute autoCreateContainerGaps: Boolean;
}

attribute GroupPanel.autoCreateGaps = true;
attribute GroupPanel.autoCreateContainerGaps = true;
attribute GroupPanel.focusable = false;

operation GroupPanel.createComponent():<<javax.swing.JComponent>> {
    panel = new <<javax.swing.JPanel>>();
    panel.setOpaque(false);
    layout = new <<org.jdesktop.layout.GroupLayout>>(panel);
    panel.setLayout(layout);
    layout.setAutocreateGaps(autoCreateGaps);
    layout.setAutocreateContainerGaps(autoCreateContainerGaps);
    //this.addComponents(panel);
    var hgroup = layout.createSequentialGroup();
    var vgroup = layout.createSequentialGroup();
    this.addColumns(panel, layout, hgroup);
    this.addRows(panel, layout, vgroup);
    layout.setHorizontalGroup(hgroup);
    layout.setVerticalGroup(vgroup);
    return panel;
}
