/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.DesktopPane;

public class InternalFrame extends Widget {
    // private
    private attribute jinternalframe: <<javax.swing.JInternalFrame>>;
    // public
    public attribute isPalette: Boolean;
    public attribute menubar: MenuBar?;
    public attribute layer: Number;
    public attribute desk: DesktopPane;
    public attribute title: String?;
    public attribute content: Widget;
    public attribute iconifiable:Boolean?;
    public attribute closable:Boolean?;
    public attribute resizable:Boolean?;
    public attribute maximizable:Boolean?;
    public attribute onClose: function();
    public operation toFront();
    public attribute selected: Boolean;
    public attribute deselected: Boolean;
}

attribute InternalFrame.focusable = false;

trigger on InternalFrame.menubar = menubar {
    jinternalframe.setJMenuBar(menubar.jmenubar);
}

trigger on (InternalFrame.selected = value) {
    if (value) {
        jinternalframe.setVisible(true);
        jinternalframe.setSelected(value);
    } 
}

trigger on (InternalFrame.deselected = value) {
    if (value) {
        jinternalframe.setVisible(false);
    }
}

trigger on (new InternalFrame) {
    closable = true;
    resizable = true;
    maximizable = true;
    iconifiable = true;
}


operation InternalFrame.createComponent() {
    jinternalframe = new <<javax.swing.JInternalFrame>>();
    if (title <> null) {
        jinternalframe.setTitle(title);
    }
    if (closable <> null) {
        jinternalframe.setClosable(closable);
    }
    if (iconifiable <> null) {
        jinternalframe.setIconifiable(iconifiable);
    }
    if (maximizable <> null) {
        jinternalframe.setMaximizable(maximizable);
    }
    if (resizable <> null) {
        jinternalframe.setResizable(resizable);
    }
    if (menubar <> null) {
        jinternalframe.setJMenuBar(menubar.jmenubar);
    }
    if (isPalette) {
        jinternalframe.putClientProperty("JInternalFrame.isPalette", true);
    }
    if (content <> null) {
        jinternalframe.setContentPane(content.getComponent());
        content.getComponent().setOpaque(true);
    }
    var self = this;
    jinternalframe.addInternalFrameListener(new <<javax.swing.event.InternalFrameListener>>() {
        operation internalFrameActivated(e) {
            for (i in self.desk.frames) {
                if (i <> self) {
                    i.selected = false;
                }
            }
            self.selected = true;
        }
        operation internalFrameDeactivated(e) {
        }
    });
    return jinternalframe;
}

trigger on InternalFrame.isPalette = value {
    jinternalframe.putClientProperty("JInternalFrame.isPalette", true);
}

trigger on (InternalFrame.iconifiable = value) {
    if (value <> null) {
        jinternalframe.setIconifiable(value);
    }
}

trigger on (InternalFrame.resizable = value) {
    if (value <> null) {
        jinternalframe.setResizable(value);
    }
}

trigger on (InternalFrame.maximizable = value) {
    if (value <> null) {
        jinternalframe.setMaximizable(value);
    }
}


trigger on (InternalFrame.closable = value) {
    if (value <> null) {
        jinternalframe.setClosable(value);
    }
}

trigger on (InternalFrame.content = value) {
    if (value <> null) {
        jinternalframe.setContentPane(value.getComponent());
        content.getComponent().setOpaque(true);
    }
}

trigger on (InternalFrame.title = value) {
    jinternalframe.setTitle(title);
}

operation InternalFrame.setBounds(b:<<java.awt.Rectangle>>) {
    jinternalframe.getDesktopPane().getDesktopManager().setBoundsForFrame(jinternalframe, b.x, b.y, b.width, b.height);
}

