/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.HorizontalAlignment;

public class PasswordField extends Widget {
    protected attribute inListener: Boolean;
    private attribute textField: <<javax.swing.JPasswordField>>;

    public attribute value: String;
    public attribute onChange: function(newValue:String);
    public attribute action: function();
    public attribute columns: Number;
    public attribute horizontalAlignment:HorizontalAlignment;
    
    /** Sets the echo character for this PasswordField, expressed as an integer value representing a Unicode character. The default value is U+2022 (\u2022), representing a bullet character. */
    public attribute echoChar: Integer;
    
    public attribute selectOnFocus:Boolean;
    public attribute selectionStart:Integer;
    public attribute selectionEnd:Integer;
    public operation selectAll();
    private attribute inSelectionUpdate:Boolean;
}

PasswordField.inSelectionUpdate = false;
PasswordField.selectionStart = 0;
PasswordField.selectionEnd = 0;
PasswordField.echoChar = 0;

trigger on (PasswordField.selectionStart = value) {
    if (inSelectionUpdate == false) {
        textField.setSelectionStart(selectionStart);
    }
}

trigger on (PasswordField.selectionEnd = value) {
    if (inSelectionUpdate == false) {
        textField.setSelectionEnd(selectionEnd);
    }
}

operation PasswordField.selectAll() {
    inSelectionUpdate = true;
    textField.selectAll();
    selectionStart = textField.getSelectionStart();
    selectionEnd = textField.getSelectionEnd();
    inSelectionUpdate = false;
}

trigger on (PasswordField.echoChar = value) {
    var string = echoChar format as <<%c>>;
    var symbol:<<char>> = string.charAt(0);
    textField.setEchoChar(symbol);
}

trigger on (PasswordField.horizontalAlignment = value) {
    textField.setHorizontalAlignment(value.id);
}

operation PasswordField.createComponent():<<javax.swing.JComponent>> {
    return textField;
}

trigger on (new PasswordField) {
    var self = this;
    
    selectOnFocus = true;
    textField = new <<javax.swing.JPasswordField>>;    
    echoChar = 0x2022; // set echoChar after we created the textfield
    var tf = textField;
    
    tf.setFont(<<javax.swing.UIManager>>.getFont("TextField.font"));

    tf.getDocument().addDocumentListener(new <<javax.swing.event.DocumentListener>>() {
                                                    operation insertUpdate(e) {
                                                        self.inListener = true;
                                                        self.value = tf.getText();
                                                        self.inListener = false;
                                                        (self.onChange)(self.value);
                                                    }
                                                    operation removeUpdate(e) {
                                                        self.inListener = true;
                                                        self.value = tf.getText();
                                                        (self.onChange)(self.value);
                                                        self.inListener = false;
                                                    }
                                                    operation changedUpdate(e) {
                                                        self.inListener = true;
                                                        self.value = tf.getText();
                                                        self.inListener = false;
                                                        (self.onChange)(self.value);
                                                    }
    });
    
    tf.addActionListener(new <<java.awt.event.ActionListener>>() {
                                    operation actionPerformed(e) {
                                        self.value = tf.getText();
                                        if (self.action <> null) {
                                              (self.action)();
                                        } else {
                                              var root = tf.getRootPane();      
                                              var but = root.getDefaultButton();
                                              if (but <> null) {
                                                  (self.onChange)(tf.getText());
                                                  but.doClick(0);
                                              }
                                        }
                                    }       
    });
    
    tf.addCaretListener(new <<javax.swing.event.CaretListener>> {
                                operation caretUpdate(e) {
                                    self.inSelectionUpdate = true;
                                    self.selectionStart = tf.getSelectionStart();
                                    self.selectionEnd = tf.getSelectionEnd();
                                    self.inSelectionUpdate = false;
                                }
    });
    
    tf.addFocusListener(new <<java.awt.event.FocusListener>> {
                                operation focusGained(e) {
                                    if (self.selectOnFocus == true) {
                                        self.selectAll();
                                    }
                                }
    });
}

trigger on (PasswordField.value = x) {
    if (not inListener) {
        textField.setText(x);
    }
}

trigger on (PasswordField.columns = n) {
    textField.setColumns(n);
}
