/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.SelectableButton;
import javafx.ui.Icon;
import javafx.ui.KeyStroke;
import javafx.ui.Insets;
import javafx.ui.HorizontalAlignment;
import javafx.ui.VerticalAlignment;

public class RadioButton extends SelectableButton {
    private attribute jradiobutton: <<javax.swing.JRadioButton>>;
    public attribute text:String;
    public attribute mnemonic: KeyStroke?;
    public attribute icon: Icon?;
    public attribute selectedIcon: Icon?;
    public attribute pressedIcon: Icon?;
    public attribute rolloverIcon: Icon?;
    public attribute rolloverSelectedIcon: Icon?;
    public attribute disabledIcon: Icon?;
    public attribute disabledSelectedIcon: Icon?;
    public attribute iconTextGap: Number?;
    public attribute contentAreaFilled: Boolean?;
    public attribute focusPainted:Boolean?;
    public attribute borderPainted:Boolean?;
    public attribute action: function();
    public attribute margin:Insets?;


    public attribute horizontalTextPosition:HorizontalAlignment?;
    public attribute verticalTextPosition:VerticalAlignment?;
    public attribute horizontalAlignment:HorizontalAlignment?;
    public attribute verticalAlignment:VerticalAlignment?;
        
}

trigger on (RadioButton.horizontalTextPosition = a) {
    jradiobutton.setHorizontalTextPosition(a.id);
    jradiobutton.revalidate();
}

trigger on (RadioButton.verticalTextPosition = a) {
    jradiobutton.setVerticalTextPosition(a.id);
    jradiobutton.revalidate();
}

trigger on (RadioButton.horizontalAlignment = a) {
    jradiobutton.setHorizontalAlignment(a.id);
    jradiobutton.revalidate();
}

trigger on (RadioButton.verticalAlignment = a) {
    jradiobutton.setVerticalAlignment(a.id);
    jradiobutton.revalidate();
}


trigger on (RadioButton.text = value) {
    jradiobutton.setText(value);
}
trigger on (RadioButton.mnemonic = value) {
    if (value <> null) {
        jradiobutton.setMnemonic(value.id);
    }
}

trigger on (new RadioButton) {
    jradiobutton = new <<javax.swing.JRadioButton>>();
    jradiobutton.setOpaque(false);
}

operation RadioButton.createComponent() {
    var self = this;
    jradiobutton.addItemListener(new <<java.awt.event.ItemListener>>() {
            operation itemStateChanged(e) {
                var i = select indexof x from x in self.buttonGroup.buttons where x == self;
                self.selected = self.jradiobutton.isSelected();
                (self.onChange)(self.selected);
                if (self.selected) {
                    self.buttonGroup.selection = i[0];
                }
            }
        });
    jradiobutton.addActionListener(new <<java.awt.event.ActionListener>>() {
            operation actionPerformed(e) {
                    (self.action)();
            }
        });
    return jradiobutton;
}

operation RadioButton.setSelected(value:Boolean) {
    jradiobutton.setSelected(value);
}

trigger on (RadioButton.focusPainted = value) {
    if (value <> null) {
        jradiobutton.setFocusPainted(value);
        jradiobutton.revalidate();
    }
}

trigger on (RadioButton.borderPainted = value) {
    if (value <> null) {
        jradiobutton.setBorderPainted(value);
        jradiobutton.revalidate();
    }
}

trigger on (RadioButton.contentAreaFilled = value) {
    if (value <> null) {
        jradiobutton.setContentAreaFilled(value);
        jradiobutton.revalidate();
    }
}

trigger on (RadioButton.icon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setIcon(icon);
    if (selectedIcon == null) {
        jradiobutton.setSelectedIcon(icon);
    }
}

trigger on (RadioButton.margin = value) {
    jradiobutton.setMargin(value.awtinsets);
    jradiobutton.revalidate();
}

trigger on (RadioButton.rolloverIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setRolloverIcon(icon);
}

trigger on (RadioButton.rolloverSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setRolloverSelectedIcon(icon);
}

trigger on (RadioButton.selectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setSelectedIcon(icon);
}

trigger on (RadioButton.pressedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setPressedIcon(icon);
}

trigger on (RadioButton.disabledIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setDisabledIcon(icon);
}

trigger on (RadioButton.disabledSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jradiobutton.setDisabledSelectedIcon(icon);
}

trigger on (RadioButton.iconTextGap = value) {
    jradiobutton.setIconTextGap(value);
}
