/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.Orientation;
import javafx.ui.SplitView;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

public class SplitPane extends Widget {
    protected attribute split:<<net.java.javafx.ui.MultiSplitPane>>;
    protected attribute root: <<net.java.javafx.ui.MultiSplitLayout$Split>>;

    public attribute orientation: Orientation;
    public attribute content: SplitView+;
    private operation getModel(): List;
}

attribute SplitPane.focusable = false;

operation SplitPane.getModel() {
    var n = select p.getSplitNode() from p in content;
    var div = null;
    var result = new ArrayList();
    for (i in n) {
        if (div <> null) {
            result.add(div);
        }
        result.add(i);
        div = new <<net.java.javafx.ui.MultiSplitLayout$Divider>>;
    }
    return result;
}

trigger on (SplitPane.orientation[old] = value) {
    if (root <> null) {
        root.setRowLayout(value == HORIZONTAL:Orientation);
    }
}

trigger on (new SplitPane) {
    orientation = HORIZONTAL:Orientation;
}

operation SplitPane.createComponent():<<javax.swing.JComponent>> {
    split = new <<net.java.javafx.ui.MultiSplitPane>>;
    split.setOpaque(false);
    root = <<net.java.javafx.ui.MultiSplitLayout$Split>>;
    root.setRowLayout(orientation == HORIZONTAL:Orientation);
    root.setWeight(1.0);
    root.setChildren(this.getModel());
    split.getMultiSplitLayout().setModel(root);
    for (p in content) {
        split.add(p.id, p.content.getComponent());
    }
    return split;
    
}

trigger on SplitPane.content[old] = value {
    var c = old.content.getComponent();
    if (c <> null) {
        split.remove(c);
    }
    split.add(value.id, value.content.getComponent());
}

trigger on (insert pane into SplitPane.content) {
    pane.splitpane = this;
    if (split <> null) {
        split.add(pane.id, pane.content.getComponent());
        root.setChildren(this.getModel());
        split.getMultiSplitLayout().setModel(root);
    }
}

trigger on (delete pane from SplitPane.content) {
    if (split <> null) {
        split.remove(indexof pane);
        root.setChildren(this.getModel());
        split.getMultiSplitLayout().setModel(root);
    }
}

