/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import java.awt.LayoutManager2;
import javax.swing.JPanel;
import java.awt.Dimension;
import net.java.javafx.ui.LayoutNotifier;
import java.lang.Math;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetListener;
import javax.swing.JViewport;
import java.awt.BorderLayout;
import net.java.javafx.ui.ScrollablePanel;
import javafx.ui.Widget;
import javafx.ui.Orientation;


class TabSlider extends Widget {
    // private
    attribute inUpdateSelection: Boolean;
    operation updateSelection(oldValue:Number, newValue:Number);
    attribute resizing: Boolean;
    attribute inSelection: Boolean;
    attribute orientation: Orientation;
    attribute panel: LayoutNotifier;

    // public
    attribute selectedIndex: Number;
    attribute tabs: SlideTab*;
    attribute slideDuration: Number;
}

attribute TabSlider.selectedIndex = -1;
attribute TabSlider.slideDuration = 700;
attribute TabSlider.orientation = VERTICAL:Orientation;
attribute TabSlider.focusable = false;

trigger on TabSlider.resizing = value {
   if (not value) {
        panel.doLayout();
        panel.repaint();
   }
}

trigger on insert t into TabSlider.tabs {
    t.slider = this;
}

trigger on SlideTab.selected = value {
    if (slider <> null) {
        if (not slider.inSelection) {
            if (value) {
                var i = select indexof x from x in slider.tabs where x == this;
                slider.selectedIndex = i[0];
            } else {
                slider.selectedIndex = -1;
            }
        }
    }
}

trigger on TabSlider.selectedIndex[old] = value {
    if (inUpdateSelection) {
        return;
    }
    if (value == -1 and sizeof tabs > 0) {
        inUpdateSelection = true;
        selectedIndex = sizeof tabs -1;
        inUpdateSelection = false;
    } 
    if (old == selectedIndex) {
        return;
    }
    this.updateSelection(old, selectedIndex);
}

operation TabSlider.updateSelection(old:Number, value:Number) {
    if (panel == null) {
        return;
    }
    var oldTab = tabs[old];
    if (oldTab <> null) {
        inSelection = true;
        oldTab.selected = false;
        inSelection = false;
    }
    var newTab = tabs[value];
    if (orientation == VERTICAL:Orientation) {
        if (newTab <> null) {
            var availHeight = panel.getHeight() - sizeof tabs*23;
            var ins = panel.getInsets();
            if (ins <> null) {
                availHeight -= ins.top + ins.bottom;
            }
            newTab.scrollable.setScrollableTracksViewportHeight(false);
            newTab.scrollable.setScrollableTracksViewportWidth(false);
            var dim = newTab.viewport.getSize();
            dim.height = availHeight;
            newTab.scrollable.setPreferredScrollableViewportSize(dim);
            if (oldTab <> null) {
                oldTab.scrollable.setScrollableTracksViewportWidth(false);
                oldTab.scrollable.setScrollableTracksViewportHeight(false);
                oldTab.scrollable.setPreferredScrollableViewportSize(oldTab.viewport.getSize());
            }
            resizing = true;
            for (i in [0..availHeight]) (dur slideDuration) {
                newTab.size = i;
                if (oldTab <> null) {
                    oldTab.size = availHeight - i;
                }
                panel.validate();
                panel.doLayout();
                panel.repaint();
                if (i == availHeight) {
                    do later {
                        newTab.scrollable.setScrollableTracksViewportHeight(true);
                        newTab.scrollable.setScrollableTracksViewportWidth(true);
                        newTab.scrollable.setPreferredScrollableViewportSize(null);
                        oldTab.scrollable.setScrollableTracksViewportHeight(true);
                        oldTab.scrollable.setScrollableTracksViewportWidth(true);
                        oldTab.scrollable.setPreferredScrollableViewportSize(null);
                        resizing = false;
                    }
                }
            }
            newTab.content.getNonScrollPaneComponent().requestFocus();
        } else if (oldTab <> null) {
            resizing = true;
            oldTab.scrollable.setScrollableTracksViewportHeight(false);
            oldTab.scrollable.setScrollableTracksViewportWidth(false);
            oldTab.scrollable.setPreferredScrollableViewportSize(oldTab.viewport.getSize());
            for (i in [oldTab.size..0]) (dur slideDuration) {
                oldTab.size = i;
                panel.validate();
                panel.doLayout();
                panel.repaint();
                if (i == 0) {
                    do later {
                        oldTab.scrollable.setScrollableTracksViewportHeight(true);
                        oldTab.scrollable.setScrollableTracksViewportWidth(true);
                        oldTab.scrollable.setPreferredScrollableViewportSize(null);
                        resizing = false;
                    }
                }
            }
        }
    } else {
        if (newTab <> null) {
            var avail = panel.getWidth() - sizeof tabs*23;
            var ins = panel.getInsets();
            var dim = newTab.viewport.getSize();
            dim.width = avail;
            if (ins <> null) {
                avail -= ins.left + ins.right;
            }
            resizing = true;
            newTab.scrollable.setScrollableTracksViewportWidth(false);
            newTab.scrollable.setScrollableTracksViewportHeight(false);
            newTab.scrollable.setPreferredScrollableViewportSize(dim);
            if (oldTab <> null) {
                oldTab.scrollable.setScrollableTracksViewportWidth(false);
                oldTab.scrollable.setScrollableTracksViewportHeight(false);
                oldTab.scrollable.setPreferredScrollableViewportSize(oldTab.viewport.getSize());
            }
            for (i in [0..avail]) (dur slideDuration) {
                newTab.size = i;
                if (oldTab <> null) {
                    oldTab.size = avail - i;
                }
                panel.validate();
                panel.doLayout();
                panel.repaint();
                if (i == avail) {
                    do later {
                        newTab.scrollable.setScrollableTracksViewportHeight(true);
                        newTab.scrollable.setScrollableTracksViewportWidth(true);
                        newTab.scrollable.setPreferredScrollableViewportSize(null);
                        oldTab.scrollable.setScrollableTracksViewportHeight(true);
                        oldTab.scrollable.setScrollableTracksViewportWidth(true);
                        oldTab.scrollable.setPreferredScrollableViewportSize(null);
                        resizing = false;
                    }
                }
            }
            newTab.content.getNonScrollPaneComponent().requestFocus();
        } else if (oldTab <> null) {
            oldTab.scrollable.setScrollableTracksViewportHeight(false);
            oldTab.scrollable.setScrollableTracksViewportWidth(false);
            oldTab.scrollable.setPreferredScrollableViewportSize(oldTab.viewport.getSize());
            resizing = true;
            for (i in [oldTab.size..0]) (dur slideDuration) {
                oldTab.size = i;
                panel.validate();
                panel.doLayout();
                panel.repaint();
                if (i == 0) {
                    do later {
                        oldTab.scrollable.setScrollableTracksViewportHeight(true);
                        oldTab.scrollable.setScrollableTracksViewportWidth(true);
                        oldTab.scrollable.setPreferredScrollableViewportSize(null);
                        resizing = false;
                    }
                }
            }
        }
    }
}

trigger on TabSlider.orientation = value {
    if (value == VERTICAL:Orientation) {
        for (i in  tabs) {
            ((RotatableWidget)i.button).rotation = 0;
        }
    } else {
        for (i in  tabs) {
            ((RotatableWidget)i.button).rotation = 90;
        }
    }
    panel.validate();
    panel.doLayout();
    panel.repaint();
}

operation TabSlider.createComponent() {
    panel = new LayoutNotifier();
    panel.setOpaque(false);
    var self = this;
    panel.setLayoutManager(new LayoutManager2() {
            operation addLayoutComponent(comp, constraint) {
            }
            operation maximumLayoutSize(container) {
                return new Dimension(MAX_INT:Number, MAX_INT:Number);
            }
            operation minimumLayoutSize(container) {
                return new Dimension(0, 0);
            }
            operation preferredLayoutSize(container) {
                if (self.orientation == VERTICAL:Orientation) {
                    var tabCount = sizeof self.tabs;
                    var comp = self.tabs[self.selectedIndex];
                    var w = 0;
                    var h = tabCount * 23;
                    for (i in self.tabs) {
                        var c = i.button.getComponent();
                        var pref = c.getPreferredSize();
                        if (pref.width > w) {
                            w = pref.width;
                        }
                        c = i.content.getComponent();
                        pref = c.getPreferredSize();
                        if (pref.width > w) {
                            w = pref.width;
                        }
                        h += i.size;
                    }
                    var ins = self.panel.getInsets();
                    if (ins <> null) {
                        w += ins.left + ins.right;
                        h += ins.top + ins.bottom;
                    } 
                    return new Dimension(w, h);
                } else {
                    var tabCount = sizeof self.tabs;
                    var comp = self.tabs[self.selectedIndex];
                    var w = tabCount * 23;
                    var h = 0;
                    for (i in self.tabs) {
                        var c = i.button.getComponent();
                        var pref = c.getPreferredSize();
                        if (pref.height > h) {
                            h = pref.height;
                        }
                        c = i.content.getComponent();
                        pref = c.getPreferredSize();
                        if (pref.height > h) {
                            h = pref.height;
                        }
                        w += i.size;
                    }
                    var ins = self.panel.getInsets();
                    if (ins <> null) {
                        w += ins.left + ins.right;
                        h += ins.top + ins.bottom;
                    } 
                    return new Dimension(w, h);
                } 
            }
            operation getLayoutAlignmentX(container) {
                return 0;
            }
            operation getLayoutAlignmentY(container) {
                return 0;
            }
            operation invalidateLayout(container) {
            }
            operation layoutContainer(container) {
                if (self.orientation == VERTICAL:Orientation) {
                    var w = self.panel.getWidth();
                    var tabCount = sizeof self.tabs;
                    var availHeight = Math.max(self.panel.getHeight() - tabCount*23, 0);
                    var x = 0;
                    var y = 0;
                    var ins = self.panel.getInsets();
                    if (ins == null) {
                        ins = new <<java.awt.Insets>>(0, 0, 0, 0);
                    }
                    x += ins.left;
                    y += ins.top;
                    w -= ins.left + ins.right;
                    if (w < 0) {
                        w = 0;
                    }
                    availHeight -= ins.top + ins.bottom;
                    var showing = self.tabs[self.selectedIndex];
                    if (showing <> null and not self.resizing) {
                        showing.size = availHeight;
                    }
                    for (i in self.tabs) {
                        var c = i.button.getComponent();
                        c.setBounds(x, y, w, 23);
                        c = i.viewport;
                        y += 23;
                        if (not self.resizing) {
                            if (i.size > 0) {
                               var dim = new Dimension(w, i.size);
                               i.viewport.getView().setSize(dim);
                            }
                        }
                        c.setBounds(x, y, w, i.size);
                        y += i.size;
                    }
                } else {
                    var tabCount = sizeof self.tabs;
                    var avail = Math.max(self.panel.getWidth() - tabCount*23, 0);
                    var x = 0;
                    var y = 0;
                    var h = self.panel.getHeight();
                    var ins = self.panel.getInsets();
                    if (ins <> null) {
                        x += ins.left;
                        y += ins.top;
                        h -= ins.top + ins.bottom+1;
                    } 
                    var showing = self.tabs[self.selectedIndex];
                    if (showing <> null and not self.resizing) {
                        showing.size = avail;
                    }
                    for (i in self.tabs) {
                        var c = i.button.getComponent();
                        c.setBounds(x, y, 23, h);
                        c = i.viewport;
                        x += 23;
                        if (not self.resizing) {
                            if (i.size > 0) {
                               i.viewport.getView().setSize(new Dimension(i.size, h));
                            }
                        }
                        c.setBounds(x, y, i.size, h);
                        x += i.size;
                    }
                }
            }
            operation removeLayoutComponent(comp) {
            }
        });
    var selection = -1;
    for (i in tabs) {
        if (i.button == null) {
            i.button = new Button(); 
            if (i.title <> null) {
                ((Button)i.button).text = i.title;
            }
            if (i.icon <> null) {
                ((Button)i.button).icon = i.icon;
            }
            if (i.icon <> null) {
                ((Button)i.button).icon = i.icon;
            }
        } 
        i.button.toolTipText = i.toolTipText;
        i.button.focusable = false;
        i.button.enabled = i.enabled;
        var dtlistener = new DropTargetListener() {
                operation dragOver(e) {
                    if (not self.resizing) {
                        i.selected = true;
                    }
                }
                operation dragEnter(e) {
                    if (not self.resizing) {
                        i.selected = true;
                    }
                }
            };
        if (i.button instanceof RotatableWidget) {
            var b = (RotatableWidget)i.button;
            b.rotation = if orientation == VERTICAL:Orientation then 0 else 90;
        }
        if (i.button instanceof ActionWidget) {
            var b = (ActionWidget)i.button;
            b.action = operation() {
                if (not self.resizing) {
                   i.selected = not i.selected;
                }
            };
        }
        new DropTarget(i.button.getComponent(), dtlistener);
        i.button.opaque = true;
        panel.add(i.button.getComponent());
        // make a non scrollable child for the viewport
        var sp = new ScrollablePanel;
        sp.setOpaque(false);
        sp.setLayout(new BorderLayout());
        sp.setScrollableTracksViewportHeight(true);
        sp.setScrollableTracksViewportWidth(true);
        sp.add(i.content.getComponent(), BorderLayout.CENTER);
        i.scrollable = sp;
        i.viewport = new JViewport();
        i.viewport.setOpaque(false);
        i.viewport.setView(sp);
        panel.add(i.viewport);
        if (i.selected) {
            selection = indexof i;
        }
    }
    do later {
        if (selection >= 0 and selection <> selectedIndex) {
            selectedIndex = selection;
        } else {
            this.updateSelection(-1, selectedIndex);
        }
    }
    return panel;
}
