/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.HorizontalAlignment;

public class TextField extends Widget {
    private attribute propertyChangeListener:<<java.beans.PropertyChangeListener>>;
    private attribute textField: <<javax.swing.JTextField>>;
    private attribute verifier: <<net.java.javafx.ui.XInputVerifierImpl>>;


    public attribute value: String;
    public attribute text: String;
    public attribute onChange: function(newValue:String)?;
    public attribute action: function()?;
    public attribute columns: Number;
    public attribute editable: Boolean;
    public attribute enableDND: Boolean;
    public attribute verify: function(newValue:String):Boolean?;
    public attribute horizontalAlignment:HorizontalAlignment;
    public attribute margin: Insets?;
    
    public attribute selectOnFocus:Boolean;
    public attribute selectionStart:Integer;
    public attribute selectionEnd:Integer;
    public operation selectAll();
    private attribute inSelectionUpdate:Boolean;
}

TextField.inSelectionUpdate = false;
TextField.selectionStart = 0;
TextField.selectionEnd = 0;

trigger on (TextField.selectionStart = value) {
    if (inSelectionUpdate == false) {
        textField.setSelectionStart(selectionStart);
    }
}

trigger on (TextField.selectionEnd = value) {
    if (inSelectionUpdate == false) {
        textField.setSelectionEnd(selectionEnd);
    }
}

operation TextField.selectAll() {
    inSelectionUpdate = true;
    textField.selectAll();
    selectionStart = textField.getSelectionStart();
    selectionEnd = textField.getSelectionEnd();
    inSelectionUpdate = false;
}

trigger on TextField.enableDND = value {
    textField.setDragEnabled(value);
}

trigger on TextField.margin = value {
    textField.setMargin(value.awtinsets);
}

trigger on (TextField.horizontalAlignment = value) {
    textField.setHorizontalAlignment(value.id);
}

operation TextField.createComponent():<<javax.swing.JComponent>> {
    return textField;
}

trigger on (TextField.editable = value) {
    textField.setEditable(value);
}

trigger on (TextField.verify = newValue) {
    if (verifier == null) {
        verifier = new <<net.java.javafx.ui.XInputVerifierImpl>>();
        var self = this;
        verifier.addInputVerifier(new <<net.java.javafx.ui.XInputVerifier>>() {
                                      operation verify(comp) {
                                          return (self.verify)(self.textField.getText());
                                      }
                                  });
        textField.setInputVerifier(verifier);
    }
}

trigger on (new TextField) {
    var self = this;
    
    editable = true;
    enableDND = true;
    selectOnFocus = true;
    text = value;
    textField = new <<javax.swing.JFormattedTextField>>();
    var tf = textField;
    
    tf.setFont(<<javax.swing.UIManager>>.getFont("TextField.font"));
    var selectionColor = <<javax.swing.UIManager>>.getColor("TextField.selectionColor");
    if (selectionColor == null) {
        var temp = new <<javax.swing.JTextField>>();
        selectionColor = temp.getSelectionColor();
        <<javax.swing.UIManager>>.put("TextField.selectionColor", selectionColor);
    }
    tf.setSelectionColor(selectionColor);
    var selectedTextColor = <<javax.swing.UIManager>>.getColor("TextField.selectedTextColor");
    if (selectedTextColor == null) {
        var temp = new <<javax.swing.JTextField>>();
        selectedTextColor = temp.getSelectedTextColor();
        <<javax.swing.UIManager>>.put("TextField.selectedTextColor", selectedTextColor);
    }
    tf.setSelectedTextColor(selectedTextColor);
    
    propertyChangeListener = new <<java.beans.PropertyChangeListener>>() {
            nodebug operation propertyChange(e:<<java.beans.PropertyChangeEvent>>) {
                if (e.getPropertyName().equals("value")) {
                      do later {
                              self.value = tf.getText();
                             (self.onChange)(tf.getText());
                       }
                }
            }
    };
    textField.addPropertyChangeListener(propertyChangeListener);
    
    textField.addActionListener(new <<java.awt.event.ActionListener>>() {
                                    operation actionPerformed(e) {
                                        self.value = tf.getText();
                                       (self.onChange)(tf.getText());
                                        if (self.action <> null) {
                                            (self.action)();
                                        } else {
                                              var root = tf.getRootPane();      
                                              var but = root.getDefaultButton();
                                              if (but <> null) {
                                                  but.doClick(0);
                                              }
                                        }
                                    }       
                                });
    
    if (true) {        
    context:<<net.java.javafx.ui.UIContext>>.addTransferHandler(textField,
                                                        :String,
                                                        new <<net.java.javafx.ui.ValueGetter>>() {
                                                            operation get() {
                                                                return self.value;
                                                            }
                                                        },
                                                        new <<net.java.javafx.ui.ValueSetter>>() {
                                                            operation set(value) {
                                                                self.value = "{value}";
                                                                
                                                            }
                                                        },
                                                        new <<net.java.javafx.ui.ValueAcceptor>>() {
                                                            operation accept(value) {
                                                                return self.enableDND;
                                                            }
                                                        },
                                                        new <<net.java.javafx.ui.VisualRepresentation>>() {
                                                            operation getComponent(value) {
                                                                var label = TextField {
                                                                    value: self.value
                                                                    border: self.border
                                                                    foreground: self.foreground
                                                                    background: self.background
                                                                    columns: self.columns
                                                                    value: value
                                                                };
                                                                return label.getComponent();
                                                            }
                                                        });
    }
    
    var documentListener = new <<javax.swing.event.DocumentListener>>() {
                                    operation insertUpdate(e:<<javax.swing.event.DocumentEvent>>) {
                                        self.text = tf.getText();
                                    }
                                    operation removeUpdate(e:<<javax.swing.event.DocumentEvent>>) {
                                        self.text = tf.getText();
                                    }
                                    operation changedUpdate(e:<<javax.swing.event.DocumentEvent>>) {
                                        self.text = tf.getText();
                                    }
    };
    tf.getDocument().addDocumentListener(documentListener);
    
    tf.addCaretListener(new <<javax.swing.event.CaretListener>> {
                                operation caretUpdate(e) {
                                    self.inSelectionUpdate = true;
                                    self.selectionStart = tf.getSelectionStart();
                                    self.selectionEnd = tf.getSelectionEnd();
                                    self.inSelectionUpdate = false;
                                }
    });
    
    tf.addFocusListener(new <<java.awt.event.FocusListener>> {
                                operation focusGained(e) {
                                    if (self.selectOnFocus == true) {
                                        self.selectAll();
                                    }
                                }
    });
}

trigger on (TextField.value = x) {
    if (x <> textField.getText()) {
        textField.setText(x);
    }
}

trigger on (TextField.columns = n) {
    textField.setColumns(n);
}

trigger on (TextField.onChange[old] = value) {
    if (old == null and value <> null) {
        textField.addPropertyChangeListener(propertyChangeListener);
    } else if (old <> null) {
        textField.removePropertyChangeListener(propertyChangeListener);
    }
}
