/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.SelectableButton;
import javafx.ui.Icon;
import javafx.ui.KeyStroke;

public class ToggleButton extends SelectableButton {
    protected attribute button: <<net.java.javafx.ui.XToggleButton>>;
    public attribute defaultButton: Boolean?;
    public attribute text: String;
    public attribute mnemonic: KeyStroke?;
    public attribute icon: Icon?;
    public attribute selectedIcon: Icon?;
    public attribute pressedIcon: Icon?;
    public attribute rolloverIcon: Icon?;
    public attribute rolloverSelectedIcon: Icon?;
    public attribute disabledIcon: Icon?;
    public attribute disabledSelectedIcon: Icon?;
    public attribute iconTextGap: Number?;
    public attribute contentAreaFilled: Boolean?;
    public attribute focusPainted:Boolean?;
    public attribute borderPainted: Boolean?;
    public attribute action: function();
}

trigger on (ToggleButton.borderPainted = value) {
    button.setBorderPainted(value);
}

operation ToggleButton.createComponent() {
    return button;
}

trigger on (ToggleButton.focusPainted = value) {
    button.setFocusPainted(value);
}

trigger on (ToggleButton.contentAreaFilled = value) {
    button.setContentAreaFilled(value);
}

trigger on (ToggleButton.icon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setIcon(icon);
    if (selectedIcon == null) {
        button.setSelectedIcon(icon);
    }
}

trigger on (ToggleButton.rolloverIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setRolloverIcon(icon);
}

trigger on (ToggleButton.rolloverSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setRolloverSelectedIcon(icon);
}

trigger on (ToggleButton.selectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setSelectedIcon(icon);
}

trigger on (ToggleButton.pressedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setPressedIcon(icon);
}

trigger on (ToggleButton.disabledIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setDisabledIcon(icon);
}

trigger on (ToggleButton.disabledSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setDisabledSelectedIcon(icon);
}

trigger on (ToggleButton.iconTextGap = value) {
    button.setIconTextGap(value);
}

trigger on (ToggleButton.text = value) {
    button.setText(value);
}

trigger on (new ToggleButton) {
    button = context:<<net.java.javafx.ui.UIContext>>.createToggleButton();
    enabled = true;
    button.setOpaque(false);
    var zb = this;
    button.addActionListener(new <<java.awt.event.ActionListener>> {
                                 operation actionPerformed(e:<<java.awt.event.ActionEvent>>) {
                                     (zb.action)();
                                 }
                             });
    var self = this;
    button.addItemListener(new <<java.awt.event.ItemListener>>() {
            operation itemStateChanged(e) {
                var i = select indexof x from x in self.buttonGroup.buttons where x == self;
                self.selected = self.button.isSelected();
                //self.onChange(self.selected);
                if (self.selected) {
                     self.buttonGroup.selection = i[0];
                }
            }
        });
}

operation ToggleButton.setSelected(value:Boolean) {
    button.setSelected(value);
}

