/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.TreeCell;
import javafx.ui.ScrollableWidget;
import javafx.ui.TreePath;
import <<net.java.javafx.ui.UIContextImpl$F3TreeCellRenderer>> as F3CellRenderer;
import <<net.java.javafx.ui.UIContextImpl$F3TreeModel>> as F3TreeModel;

public class Tree extends ScrollableWidget {
    private attribute cellRenderer: F3CellRenderer;
    private attribute inSelectionChange: Boolean;
    private attribute inModel: Number;
    private operation selectionUpdate(tp:TreePath);
    private operation fireRowInserted(row:TreeCell, i:Number);
    private operation fireRowDeleted(row:TreeCell, i:Number);
    private operation fireTreeStructureChanged(row:TreeCell);
    private operation fireNodeValueChanged(row:TreeCell);
    private attribute treeSelectionListener: <<javax.swing.event.TreeSelectionListener>>;
    private attribute listeners: <<javax.swing.event.TreeModelListener>>*;
    private attribute tree: <<javax.swing.JTree>>;
    private attribute treemodel: F3TreeModel;
    public attribute selection: TreePath?;
    public attribute selectedValue:*;
    public attribute leadSelectionPath: **;
    public attribute onSelectionChange:function()?;
    public attribute showRootHandles:Boolean?;
    public attribute rootVisible:Boolean?;
    public attribute root: TreeCell;
    public attribute rowHeight: Number;
    public attribute cellBackground: AbstractColor?;
    public attribute cellForeground: AbstractColor?;
    public attribute selectedCellBackground: AbstractColor?;
    public attribute selectedCellForeground: AbstractColor?;
}

trigger on Tree.cellBackground = value {
    cellRenderer.setCellBackground(value.getColor());
}

trigger on Tree.cellForeground = value {
    cellRenderer.setCellForeground(value.getColor());
}

trigger on Tree.selectedCellBackground = value {
    cellRenderer.setSelectedCellBackground(value.getColor());

}

trigger on Tree.selectedCellForeground = value {
    cellRenderer.setSelectedCellForeground(value.getColor());
}

trigger on insert e into Tree.leadSelectionPath {
   if (inSelectionChange) {return;}
   if (sizeof leadSelectionPath > 0 and root.value == leadSelectionPath[0]) {
       var tp = new <<javax.swing.tree.TreePath>>(root);
       var p = root;
       for (i in [1..<sizeof leadSelectionPath]) {
           var cell = p.cells[c|c.value == leadSelectionPath[i]];
           if (cell <> null) {
               tp = tp.pathByAddingChild(cell);
           } else {
               break;
           }
           p = cell[0];
       }
       tree.setLeadSelectionPath(tp);
   }
}

trigger on Tree.rowHeight = value {
    tree.setRowHeight(value);
}

trigger on Tree.leadSelectionPath[oldValue] = newValue {
   if (inSelectionChange) {return;}
   if (sizeof leadSelectionPath > 0 and root.value == leadSelectionPath[0]) {
       var tp = new <<javax.swing.tree.TreePath>>(root);
       var p = root;
       for (i in [1..<sizeof leadSelectionPath]) {
           var cell = p.cells[c|c.value==leadSelectionPath[i]];
           if (cell <> null) {
               tp = tp.pathByAddingChild(cell[0]);
           } else {
               break;
           }
           p = cell[0];
       }
       tree.setLeadSelectionPath(tp);
   }
}

trigger on delete e from Tree.leadSelectionPath {
   if (inSelectionChange) {return;}
   if (sizeof leadSelectionPath > 0 and root.value == leadSelectionPath[0]) {
       var tp = new <<javax.swing.tree.TreePath>>(root);
       var p = root;
       for (i in [1..<sizeof leadSelectionPath]) {
           var cell = p.cells[c|c.value==leadSelectionPath[i]];
           if (cell <> null) {
               tp = tp.pathByAddingChild(cell[0]);
           } else {
               break;
           }
           p = cell[0];
       }
       tree.setLeadSelectionPath(tp);
   }
}

operation Tree.createView():<<javax.swing.JComponent>> {
    return tree;
}

trigger on (Tree.rootVisible = value) {
    tree.setRootVisible(value);
}

trigger on (Tree.showRootHandles = value) {
    tree.setShowsRootHandles(value);
}


trigger on (new Tree) {
    tree = context:<<net.java.javafx.ui.UIContext>>.createTree();
    var self = this;
    cellRenderer = (F3CellRenderer)tree.getCellRenderer();    
    tree.setModel(null);
    var selectionCount = 0;
    /*
    treeSelectionListener = new <<javax.swing.event.TreeSelectionListener>>() {
        operation valueChanged(e:<<javax.swing.event.TreeSelectionEvent>>) {
            var count = ++selectionCount;
            for (unitinterval r in dur 200 fps 100 while count == selectionCount) {
                if (r == 1 and count == selectionCount) {
                    self.inSelectionChange = true;
                    var old = e.getOldLeadSelectionPath();
                    var p = e.getNewLeadSelectionPath();
                    var oldP = TreePath {
                    };
                    var newP = TreePath {
                    };
                    for (i in old.getPath()) {
                        var cell = (TreeCell)i;
                        insert cell.value into oldP.nodes;
                    }
                    for (i in p.getPath()) {
                        var cell = (TreeCell)i;
                        insert cell.value into newP.nodes;
                    }
                    if (newP.nodes <> self.selection.nodes) {
                        self.selection = newP;
                    }
                    var selectedCell = (TreeCell)p.getLastPathComponent();
                    var oldLead = (TreeCell)old.getLastPathComponent();
                    oldLead.selected = false;
                    selectedCell.selected = true;
                    self.inSelectionChange = false;
                }
            }
        }
    };
    */
}

trigger on (Tree.selection = newSelection) {
    selectedValue = selection.nodes[sizeof selection.nodes-1];
    if (treemodel <> null) {
        this.selectionUpdate(newSelection);
    }
}

trigger on Tree.inSelectionChange = newValue {
     if (not newValue) {
        tree.repaint();
     }
}

operation Tree.selectionUpdate(newSelection:TreePath) {
    if (inSelectionChange) {return;}
    inSelectionChange = true;
    var newPath = newSelection.nodes;
    var node = [root];
    var path:TreeCell* = [];
    //tree.getSelectionModel().removeTreeSelectionListener(treeSelectionListener);
    for (x in newPath) {
        node = node[n|n.value == x];
        if (node <> null) {
            insert node[0] into path;
            node = node.cells;
        } else {
            break;
        }
    }
    if (node <> null and sizeof path > 0) {
        var tp = new <<javax.swing.tree.TreePath>>(path[0]);
        for (i in [1..<sizeof path]) {
            tp = tp.pathByAddingChild(path[i]);
        }
        tree.clearSelection();
        tree.setSelectionPath(tp);
        tree.makeVisible(tp);
    }
    //tree.getSelectionModel().addTreeSelectionListener(treeSelectionListener);
    inSelectionChange = false;
}

trigger on (Tree.root[old] = newValue) {
    var rootSelected = root.selected;
    newValue.tree = this;
    if (treemodel == null) {
        treemodel = new F3TreeModel(this);
        tree.setModel(treemodel);
        treeSelectionListener = treemodel;
        tree.getSelectionModel().addTreeSelectionListener(treeSelectionListener);
    } else {
        this.fireTreeStructureChanged(root);
    }
    this.selectionUpdate(selection);
    if (rootSelected) {
        var p:TreeCell* = [];
        var r = root;
        while (r <> null) {
            insert r into p;
            r = r.parent;
        }
        var tp = new <<javax.swing.tree.TreePath>>(p);
        tree.addSelectionPath(tp);
    }
}

operation Tree.fireTreeStructureChanged(row:TreeCell) {
    if (true or sizeof listeners > 0) {
        var path:TreeCell* = [];
        var r = row;
        while (r <> null) {
            insert r into path;
            r = r.parent;
        }
        var event = new <<javax.swing.event.TreeModelEvent>>(this.treemodel,
                                                             reverse path,
                                                             null,
                                                             null);

        /*
        var ls:<<javax.swing.event.TreeModelListener>>* = [];
        insert listeners into ls;
        for (j in ls) {
            j.treeStructureChanged(event);
        }
        */
        treemodel.fireTreeStructureChanged(event);
    }
}


operation Tree.fireNodeValueChanged(row:TreeCell) {
    if (true or sizeof listeners > 0) {
        var path:TreeCell* = [];
        var r = row.parent;
        while (r <> null) {
            insert r into path;
            r = r.parent;
        }
        if (path == null) { 
           return;
        }
        var i = select indexof x from x in row.parent.cells where x == row;
        var event = new <<javax.swing.event.TreeModelEvent>>(this.treemodel,
                                                             reverse path,
                                                             i,
                                                             row);
        treemodel.fireTreeNodesChanged(event);
        /*
        var ls:<<javax.swing.event.TreeModelListener>>* = [];
        insert listeners into ls;
        for (j in ls) {
            j.treeNodesChanged(event);
        }
        */
    }
}


operation Tree.fireRowInserted(row:TreeCell, i:Number) {
    if (true or sizeof listeners > 0) {
        var path:TreeCell* = [];
        var r = row.parent;
        while (r <> null) {
            insert r into path;
            r = r.parent;
        }
        var event = new <<javax.swing.event.TreeModelEvent>>(this.treemodel,
                                                             reverse path,
                                                             i,
                                                             row);
        treemodel.fireTreeNodesInserted(event);
        /*
        var ls:<<javax.swing.event.TreeModelListener>>* = [];
        insert listeners into ls;
        for (j in ls) {
            j.treeNodesInserted(event);
        }
        */
    }
    if (row.selected) {
        var p:TreeCell* = [];
        var r = row;
        while (r <> null) {
            insert r into p;
            r = r.parent;
        }
        var tp = new <<javax.swing.tree.TreePath>>(p);
        tree.addSelectionPath(tp);
    }
}

operation Tree.fireRowDeleted(row:TreeCell, i:Number) {
    if (true or sizeof listeners > 0) {
        var path:TreeCell* = [];
        var r = row.parent;
        while (r <> null) {
            insert r into path;
            r = r.parent;
        }
        var event = new <<javax.swing.event.TreeModelEvent>>(this.treemodel,
                                                             reverse path,
                                                             i,
                                                             row);
        treemodel.fireTreeNodesRemoved(event);
        /*
        var ls:<<javax.swing.event.TreeModelListener>>* = [];
        insert listeners into ls;
        for (j in ls) {
            j.treeNodesRemoved(event);
        }
        */
    }
}
