/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.Canvas;

/** Abstract interface of objects that appear in a canvas. */

public class CanvasElement {
    /** The containing element of this element. */
    protected attribute parentCanvasElement: CanvasElement?;
    /** Convenience method to obtain the containing canvas. */
    protected operation getCanvas(): Canvas;
    /** Returns the canvas element that contains this element */
    public operation getContainer(): Container;
    protected operation onSetCanvas(canvas:Canvas);
    /** raise this element above its next sibling */
    public operation raise();
    /** raise this element below its previous sibling */
    public operation lower();
    /** move this element after all of its following siblings */
    public operation toFront();
    /** move this element before all of its previous siblings */
    public operation toBack();
    function hasParent(): Boolean;
    ...
}

function CanvasElement.hasParent() = parentCanvasElement <> null;

operation CanvasElement.getContainer() {
    var p = parentCanvasElement;
    while (p <> null) {
        if (p instanceof Container) {
            return (Container)p;
        }
    }
    return null;
}

operation CanvasElement.raise() {
    if (this instanceof Node) {
        this.getContainer().raiseNode((Node)this);
    }
}

operation CanvasElement.lower() {
    if (this instanceof Node) {
        this.getContainer().lowerNode((Node)this);
    }
}

operation CanvasElement.toFront() {
    if (this instanceof Node) {
        this.getContainer().moveNodeToFront((Node)this);
    }
}

operation CanvasElement.toBack() {
    if (this instanceof Node) {
        this.getContainer().moveNodeToBack((Node)this);
    }
}
