/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Node;
import net.java.javafx.jazz.ZClipGroup;
import net.java.javafx.jazz.ZGroup;
import javafx.ui.canvas.Shape;
import net.java.javafx.ui.F3NodeListener;

/** 
 * A Group node that provides an arbitrary clipping region. Only the subset
 * of its content that intersects its <code>shape</code> attribute 
 * will be painted.
 */

public class Clip extends Node, Container {
    private attribute cg: ZClipGroup;
    private attribute childGroup: ZGroup;

    /** Sets the shape that defines the visible region of the content*/
    public attribute shape: Node?;
    /** The content of this node */
    public attribute content: Node*;
    /** If true then clipping will be implemented using alpha composites onto an intermediate image. Defaults to false **/
    public attribute antialias: Boolean?;
}

attribute Clip.antialias = false;

trigger on Clip.antialias = value {
    if (cg <> null) {
        cg.setSmooth(value);
    }
}

trigger on Clip.shape = value {
    value.parentCanvasElement = this;
    if (cg <> null) {
        cg.setClipNode(value.getNode());
    }
}

trigger on insert c into Clip.content {
    c.parentCanvasElement = this;
    if (cg <> null) {
        childGroup.addChild(indexof c, c.getNode());
    }
}

trigger on Clip.content[old] = newValue {
    newValue.parentCanvasElement = this;
    if (cg <> null) {
        if (old <> null) {
            childGroup.removeChild(indexof old);
        }
        childGroup.addChild(indexof newValue, newValue.getNode());
    }

}

trigger on delete c from Clip.content {
    if (cg <> null) {
        childGroup.removeChild(indexof c);
    }

}

operation Clip.createNode() {
    cg = new ZClipGroup();
    cg.setClipVisible(false);
    cg.setClipNode(shape.getNode());
    cg.setSmooth(antialias);
    childGroup = new ZGroup();
    cg.addChild(childGroup);
    for (i in content) {
        childGroup.addChild(i.getNode());
    }
    return cg;
}

operation Clip.raiseNode(n:Node) {
    var i = select indexof x from x in content where x == n;
    if (i == sizeof content -1) {
        return;
    }
    delete content[i];
    insert n after content[i];
}
operation Clip.lowerNode(n:Node) {
    var i = select first indexof x from x in content where x == n;
    if (i == 0) {
        return;
    }
    delete content[i];
    insert n before content[i-1];
}

operation Clip.moveNodeToFront(n:Node) {
    var i = select indexof x from x in content where x == n;
    delete content[i];
    insert n as last into content;
}

operation Clip.moveNodeToBack(n:Node) {
    var i = select indexof x from x in content where x == n;
    delete content[i];
    insert n as first into content;
}
