/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.PathElement;
import java.awt.geom.GeneralPath;

/**
 * Adds a curved segment, defined by three new points, to the path by
 * drawing a B&eacute;zier curve that intersects both the current
 * coordinates and the coordinates (x3,&nbsp;y3), using the    
 * specified points (x1,&nbsp;y1) and (x2,&nbsp;y2) as
 * B&eacute;zier control points.
 */

public class CurveTo extends PathElement {
    public attribute smooth: Boolean;

    /** 
     * the x coordinate of the first B&eacute;ezier
     * control point
     */
    public attribute x1: Number;

    /** 
     * the y coordinate of the first B&eacute;ezier
     * control point
     */
    public attribute y1: Number;

    /** 
     * the x coordinate of the second B&eacute;ezier
     * control point
     */
    public attribute x2: Number;

    /** 
     * the y coordinate of the second B&eacute;ezier
     * control point
     */
    public attribute y2: Number;

    /** 
     * the x coordinate of the final endpoint
     */
    public attribute x3: Number;

    /** 
     * the y coordinate of the final endpoint
     */
    public attribute y3: Number;
}

operation CurveTo.addTo(gp:GeneralPath) {
    if (smooth) {
        if (absolute) {
            gp.curveTo(path.currentX * 2 - path.xCenter,
                       path.currentY * 2 - path.yCenter,
                       x2, y2, x3, y3);
                       
            path.xCenter = x2;
            path.yCenter = y2;
            
            path.currentX = x3;
            path.currentY = y3;
        } else {
            gp.curveTo(path.currentX * 2 - path.xCenter,
                       path.currentY * 2 - path.yCenter,
                       path.currentX + x2, 
                       path.currentY + y2, 
                       path.currentX + x3, 
                       path.currentY + y3);
                       
            path.xCenter = path.currentX + x2;
            path.yCenter = path.currentY + y2;
            
            path.currentX += x3;
            path.currentY += y3;
        }
        
        return;
    }
    
    // Not smooth
    if (absolute) {
    	path.xCenter = x2;
    	path.yCenter = y2;
    	
    	path.currentX = x3;
    	path.currentY = y3;
    	
        gp.curveTo(x1, y1, x2, y2, x3, y3);
    } else {
    	path.xCenter = path.currentX + x2;
    	path.yCenter = path.currentY + y2;
    	
    	path.currentX += x3;
    	path.currentY += y3;
    	
        var pt = gp.getCurrentPoint();
        gp.curveTo(x1 + pt.x, 
                   y1 + pt.y,
                   x2 + pt.x,
                   y2 + pt.y,
                   x3 + pt.x,
                   y3 + pt.y);
    }
}

trigger on CurveTo.x1 = value {
    path.buildPath();
}

trigger on CurveTo.x2 = value {
    path.buildPath();
}

trigger on CurveTo.x3 = value {
    path.buildPath();
}

trigger on CurveTo.y1 = value {
    path.buildPath();
}

trigger on CurveTo.y2 = value {
    path.buildPath();
}

trigger on CurveTo.y3 = value {
    path.buildPath();
}
