/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Node;
import net.java.javafx.jazz.ZGroup;
import net.java.javafx.jazz.util.ZBounds;


/**
 * Non-visual node that defines a new coordinate space for its child nodes.
 */
public class Group extends Node, Container {
    /** The child members of this group */
    protected attribute _group: ZGroup?;
    public attribute content: Node*;
}

trigger on insert c into Group.content {
    c.parentCanvasElement = this;
    if (_group <> null) {
        _group.addChild(indexof c, c.getNode());
    }
}

trigger on Group.content[old] = newValue {
    if (newValue == null) {
        //println(" assign null");
        return;
    }
    newValue.parentCanvasElement = this;
    if (_group <> null) {
        if (old <> null) {
           try {
               _group.removeChild(indexof old);
           } catch (e) {
               //println("remove: {e} old={old}");
           }
        }
        _group.addChild(indexof newValue, newValue.getNode());
    }
    if (old.parentCanvasElement == this) {
        old.parentCanvasElement = null;
    }
}

trigger on delete c from Group.content {
    if (_group <> null and c <> null) {
        try {
           _group.removeChild(indexof c);
        } catch (e) {
             //println("e={e}");
        }
    }
    if (c.parentCanvasElement == this) {
        c.parentCanvasElement = null;
    }
}

operation Group.createNode() {
    _group  = new ZGroup();
    _group.setChildrenPickable(true);
    for (i in content) {
        _group.addChild(i.getNode());
    }
    //group.putClientProperty("f3", this);
    return _group;
}

operation Group.raiseNode(n:Node) {
    var i = select indexof x from x in content where x == n;
    if (i == sizeof content -1) {
        return;
    }
    delete content[i];
    insert n after content[i];
}
operation Group.lowerNode(n:Node) {
    var i = select first indexof x from x in content where x == n;
    if (i == 0) {
        return;
    }
    delete content[i];
    insert n before content[i-1];
}

operation Group.moveNodeToFront(n:Node) {
    if (content[sizeof content-1] == n) {
        return;
    }
    var i = 0;
    for (c in content) {
       if (c == n) {
           i = indexof c;
           break;
       }
    }
    delete content[i];
    insert n as last into content;
}

operation Group.moveNodeToBack(n:Node) {
    if (content[0] == n) {
        return;
    }
    var i = 0;
    for (c in content) {
       if (c == n) {
           i = indexof c;
           break;
       }
    }
    delete content[i];
    insert n as first into content;
}
