/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Node;
import javafx.ui.Image;
import net.java.javafx.jazz.component.ZImage;
import net.java.javafx.jazz.component.ZImage$ZImageListener as ZImageListener;
import net.java.javafx.jazz.ZVisualLeaf;
import java.awt.Dimension;
import javafx.ui.Stretch;
import javafx.ui.StretchDirection;
import java.awt.Image as JImage;
/**
 * A node that contains an Image. 
 */

public class ImageView extends Node {
    private attribute awtImage: JImage?;
    private attribute zimage: ZImage;
    private attribute needsScaling: Boolean;
    public attribute image: Image?;
    public attribute animated: Boolean;
    public attribute imageOpacity: Number;
    public attribute loaded: Boolean;
    public attribute size: Dimension?;
    public attribute stretch: Stretch?;
    public attribute stretchDirection: StretchDirection?;
    public function downloadPercentage(): Number;
    operation getImage();
    operation resizeImage(im:JImage): JImage;
    public attribute preload: Boolean;
    public attribute antialias: Boolean;
    public attribute accelerate: Boolean;
    public attribute drawImmediately: Boolean;
}

attribute ImageView.imageOpacity = 1.0;
attribute ImageView.antialias = true;
attribute ImageView.awtImage = bind image.getImage();
attribute ImageView.drawImmediately = true;

function ImageView.downloadPercentage() {
    var result = if image.totalDownloadSize == 0 then 0 else image.totalDownloaded/image.totalDownloadSize;
    return if result == null then 0 else result*100;
}
   


trigger on ImageView.size = value {
    if (zimage <> null) {
        this.getImage();
    }
}

trigger on new ImageView {
    zimage = new ZImage();
    var self = this;
    zimage.setImageListener(new ZImageListener() {
            operation imageUpdate(done:Boolean) {
                self.loaded = done;
            }
        });
}

operation ImageView.createNode() {
/*
    zimage = new ZImage();
    zimage.setImageOpacity(imageOpacity);
    //zimage.putClientProperty("f3", this);
    var self = this;
    zimage.setAntialias(antialias);
    zimage.setAccelerated(accelerate);
    zimage.setIncrementalDrawing(drawImmediately);
    zimage.setAnimated(animated);
    zimage.setImageListener(new ZImageListener() {
            operation imageUpdate(done:Boolean) {
                self.loaded = done;
            }
        });
    this.getImage();
*/
    return new ZVisualLeaf(zimage);
}

trigger on ImageView.antialias = value {
     zimage.setAntialias(value);
}

trigger on ImageView.drawImmediately = value {
     zimage.setIncrementalDrawing(drawImmediately);
}

trigger on ImageView.accelerate = value {
     zimage.setAccelerated(value);
}

trigger on ImageView.animated = value {
    zimage.setAnimated(value);
}

trigger on ImageView.imageOpacity = value {
    zimage.setImageOpacity(value);
}

trigger on ImageView.awtImage = value {
    if (zimage <> null) {
        this.getImage();
    }
}

operation ImageView.resizeImage(im:JImage) {
    if (im == null) {
        return null;
    }
    do {
        new <<javax.swing.ImageIcon>>(im);
    }
    var w = (Number)im.getWidth(null);
    var h = (Number)im.getHeight(null);
    var thumb = im;
    if (size.width < size.height) {
        var ratio = w/h;
        while (true) {
            w /= 2;
            if (w < size.width) {
                w = size.width;
            }
            var temp = new <<java.awt.image.BufferedImage>>(w,  (w / ratio).intValue(), <<java.awt.image.BufferedImage>>.TYPE_INT_ARGB);
            var g2 = temp.createGraphics();
            g2.setRenderingHint(<<java.awt.RenderingHints>>.KEY_INTERPOLATION,
                                <<java.awt.RenderingHints>>.VALUE_INTERPOLATION_BILINEAR);
            g2.drawImage(thumb, 0, 0, temp.getWidth(), temp.getHeight(), null);
            g2.dispose();
            thumb = temp;
            if (w == size.width) {
                break;
            }
        } 
    } else {
        var ratio = h/w;
        while (true) {
            h /= 2;
            if (h < size.height) {
                h = size.height;
            }
            var temp = new <<java.awt.image.BufferedImage>>((h / ratio).intValue(), h, <<java.awt.image.BufferedImage>>.TYPE_INT_ARGB);
            var g2 = temp.createGraphics();
            g2.setRenderingHint(<<java.awt.RenderingHints>>.KEY_INTERPOLATION,
                                <<java.awt.RenderingHints>>.VALUE_INTERPOLATION_BILINEAR);
            g2.drawImage(thumb, 0, 0, temp.getWidth(), temp.getHeight(), null);
            g2.dispose();
            thumb = temp;
            if (h == size.height) {
                break;
            }
        } 
    }
    return thumb;
}

operation ImageView.getImage() {
    if (image <> null and size <> null) {
        loaded = false;
        var curImage = image;
        var im = curImage.getImage();
        if (preload) {
            im = resizeImage(im);
            loaded = zimage.setImage(im);
        } else {
            do later {
                im = resizeImage(im);
                if (curImage == image) {
                    //println("setting resized image");
                    loaded = zimage.setImage(im);
                }
            }
        }
    } else {
        loaded = false;
        var im = awtImage;
        zimage.setImage(im);
        zimage.reshape();
    }
}
