/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Shape;
import javafx.ui.canvas.SizeableCanvasElement;
import net.java.javafx.jazz.component.ZRectangle;

/**
 * The <code>Rect</code> class defines a rectangle with
 * possibly rounded corners defined by a location (x,&nbsp;y), a
 * dimension (width&nbsp;&nbsp;height), and the width and height of an arc 
 * (arcWidth, arcHeight) with which to round the corners.
 * 
 */
public class Rect extends Shape, SizeableCanvasElement {
    protected attribute zrect: ZRectangle;
    /** The x coordinate of this rectangle's location. */
    public attribute x: Number;
    /** The y coordinate of this rectangle's location. */
    public attribute y: Number;
    /** The width of this rectangle. */
    public attribute width: Number;
    /** The height of this rectangle. */
    public attribute height: Number;
    /** The width of the corner arc of this rectangle. */
    public attribute arcWidth: Number;
    /** The height of the corner arc of this rectangle. */
    public attribute arcHeight: Number;
}



operation Rect.setSize(w:Number, h:Number) {
    width = w;
    height = h;
}

operation Rect.onSizeToFitCanvas(value:Boolean) {
    if (parentCanvasElement <> null) {
        var canvas = this.getCanvas();
        if (value) {
            insert this into canvas.sizeToFitList;
       } else {
            delete canvas.sizeToFitList[e|e==this];
        }
    }
}

trigger on Rect.x = value {
    zrect.setRect(x, y, width, height, arcWidth, arcHeight);
}

trigger on Rect.y = value {
    zrect.setRect(x, y, width, height, arcWidth, arcHeight);
}

trigger on Rect.width = value {
    zrect.setRect(x, y, width, height, arcWidth, arcHeight);
}

trigger on Rect.height = value {
    zrect.setRect(x, y, width, height, arcWidth, arcHeight);
}

trigger on Rect.arcWidth = value {
    zrect.setRect(x, y, width, height, arcWidth, arcHeight);
}

trigger on Rect.arcHeight = value {
    zrect.setRect(x, y, width, height, arcWidth, arcHeight);
}

operation Rect.createShape() {
    zrect = new ZRectangle(x, y, width, height, arcWidth, arcHeight);
    return zrect;
}
