/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.XY;
import javafx.ui.canvas.Layout;
import javafx.ui.canvas.GridAxis;
import net.java.javafx.jazz.ZGridLayoutManager;
import net.java.javafx.jazz.ZLayoutGroup;

class ScalingGridLayout extends Layout {
    attribute layoutmgr: ZGridLayoutManager;
    attribute axis: GridAxis;
    attribute cells: Number;
    attribute width: Number;
    attribute height: Number;
    attribute hgap: Number;
    attribute vgap: Number;
    attribute startPoint: XY;
    attribute invertChildren: Boolean;
}

attribute ScalingGridLayout.axis = ROW:GridAxis;
attribute ScalingGridLayout.cells = 1;
attribute ScalingGridLayout.width = 50.0;
attribute ScalingGridLayout.height = 50.0;
attribute ScalingGridLayout.hgap = 0.0;
attribute ScalingGridLayout.vgap = 0.0;
attribute ScalingGridLayout.startPoint = XY { x: 0.0, y: 0.0 };
attribute ScalingGridLayout.invertChildren = false;

operation ScalingGridLayout.createNode() {
    layoutgroup = new ZLayoutGroup();
    layoutmgr = new ZGridLayoutManager(axis.id, cells, width, height, hgap, vgap
				       , startPoint.x, startPoint.y);
    layoutgroup.setLayoutManager(layoutmgr);
    for (i in content) {
	layoutgroup.addChild(i.getNode());
    }
    return layoutgroup;
}

trigger on ScalingGridLayout.axis = value {
    if (layoutmgr <> null) {
	layoutmgr.setGridAxis(axis.id);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.cells = value {
    if (layoutmgr <> null) {
	layoutmgr.setNumCells(cells);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.width = value {
    if (layoutmgr <> null) {
	layoutmgr.setCellWidth(width);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.height = value {
    if (layoutmgr <> null ) {
	layoutmgr.setCellHeight(height);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.hgap = value {
    if (layoutmgr <> null) {
	layoutmgr.setHorizontalSpacing(hgap);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.vgap = value {
    if (layoutmgr <> null) {
	layoutmgr.setVerticalSpacing(vgap);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.startPoint = value {
    if (layoutmgr <> null) {
	layoutmgr.setLayoutStartPoint(startPoint.x, startPoint.y);
	layoutgroup.invalidate();
    }
}

trigger on ScalingGridLayout.invertChildren = value {
    if (layoutmgr <> null) {
	layoutmgr.setInvertChildren(invertChildren);
	layoutgroup.invalidate();
    }
}
