/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Shape;
import net.java.javafx.jazz.component.ZPolygon;
import java.lang.Math;

/**
 * A shape which describes a star by means of a center point (cx, cy),
 * an inner radius (rin), an outer radius (rout), a count of points
 * (points), and an angle at which to draw the first point (startAngle).
 */

public class Star extends Shape {

    private attribute zpolygon: ZPolygon;
    private operation updatePolygon();

    /** The x coordinate of the center point of this star. */
    public attribute cx: Number;
    /** The y coordinate of the center point of this star. */
    public attribute cy: Number;
    /** The inner radius of this star. */
    public attribute rin: Number;
    /** The outer radius of this star. */
    public attribute rout: Number;
    /** The number of points this star has.*/
    public attribute points: Number;
    /** The angle at which to draw the first point. */
    public attribute startAngle: Number; // in degrees
}

operation Star.updatePolygon() {
    if (zpolygon <> null) {
        var angle = Math.toRadians(startAngle);
        var dth = Math.PI/points;
        var xs:Number* = [];
        var ys:Number* = [];
        for (i in [0..points*2-1]) {
            insert cx + (if i %2 == 0 then rin else rout)*Math.cos(angle+dth* i) into xs;
            insert cy + (if i %2 == 0 then rin else rout)*Math.sin(angle+dth* i) into ys;
        }
        zpolygon.setCoords(xs, ys);
    }
}

trigger on Star.cx = value {
    this.updatePolygon();
}

trigger on Star.cy = value {
    this.updatePolygon();
}

trigger on Star.rin = value {
    this.updatePolygon();
}

trigger on Star.rout = value {
    this.updatePolygon();
}

trigger on Star.startAngle = value {
    this.updatePolygon();
}

trigger on Star.points = value {
    this.updatePolygon();
}

operation Star.createShape() {
    zpolygon = new ZPolygon();
    this.updatePolygon();
    return zpolygon;
}
