/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.Node;
import javafx.ui.canvas.SizeableCanvasElement;
import net.java.javafx.jazz.component.ZSwing;
import net.java.javafx.jazz.ZVisualLeaf;
import javafx.ui.Widget;
import java.awt.Dimension;
import javax.swing.event.ChangeListener;

/**
 * A Canvas node that contains a Widget (Swing component). The contained widget 
 * will be sized to its preferred size.
 */

public class View extends Node, SizeableCanvasElement {
    private attribute zswing: ZSwing;
    /** If true, the graphics context used to paint the widget will be anti-aliased. Defaults to false */
    public attribute antialias: Boolean;
    /** If true, the graphics context used to paint the widget will use anti-aliasing for text. Defaults to false.*/
    public attribute antialiasText: Boolean;
    /** If true, the graphics context used to paint the widget will use fractional metrics, otherwise integer metrics will be used. Defaults to false.*/
    public attribute fractionalMetrics: Boolean;
    /** The widget (Swing component) contained in this node */
    public attribute content: Widget;
    public attribute size: Dimension?;
    public attribute baseline: Number;
}

attribute View.antialias = false; 
attribute View.antialiasText = false; 

operation View.setSize(w: Number, h: Number) {
   zswing.setSize(w, h);
}

trigger on View.size = value {
    if (value <> null) {
        this.setSize(value.width, value.height);
    }
}

trigger on View.visible = value {
     zswing.getComponent().setVisible(value);
}

operation View.onSizeToFitCanvas(value: Boolean) {
    zswing.setSizeToFitCanvas(value);
}

operation View.createNode() {
    var canvas = this.getCanvas().zcanvas;
    zswing = new ZSwing(canvas, content.getComponent());
    zswing.setSizeToFitCanvas(sizeToFitCanvas);
    zswing.setAntialias(antialias);
    zswing.setAntialiasText(antialiasText);
    zswing.setFractionalMetrics(fractionalMetrics);
    if (size <> null) {
        this.setSize(size.width, size.height);
    }
    var self = this;
    baseline = zswing.getBaseline();
    zswing.setChangeListener(new ChangeListener() {
	    operation stateChanged(e) {
		self.baseline = self.zswing.getBaseline();
	    }
	});
    return new ZVisualLeaf(zswing);
}


trigger on View.antialias = value {
   zswing.setAntialias(value);
}

trigger on View.antialiasText = value {
   zswing.setAntialiasText(value);
}

trigger on View.fractionalMetrics = value {
   zswing.setFractionalMetrics(value);
}

trigger on View.content = value {
   zswing.setComponent(value.getComponent());
}

