/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package net.java.javafx.typeImpl;
import net.java.javafx.typeImpl.F3WebStart$F3AppManager as F3AppManager;
import net.java.javafx.typeImpl.F3WebStart$F3App as F3App;
import net.java.javafx.typeImpl.F3WebStart$F3AppListener as F3AppListener;
import net.java.javafx.typeImpl.F3WebStart$F3ConsoleView as F3ConsoleView;
import javafx.ui.*;
import javafx.ui.canvas.*;
import java.lang.System;
import java.io.File;
import java.awt.Font;

class Download {
    attribute active: Boolean;
    attribute app: F3App;
    attribute url: String;
    attribute progress: Number;
    attribute total: Number;
    attribute cancelled: Boolean;
}

class AppDownload {
    attribute total: Number;
    attribute progress: Number;
    attribute app: F3App?;    
    attribute done: Boolean;
    attribute download: Download?;
}

public class AppModel {
    attribute totalProgress: Number;
    attribute hideDownloads: Boolean;
    attribute profiling: Boolean;
    attribute apps: F3App*;
    attribute appDownloads: AppDownload*;
    attribute selection: Number?;
    attribute downloads: Download*;
    attribute cachedFiles: String*;
    attribute cachedImageFiles: String*;
    attribute selectedCachedImageFile: Number?;
    attribute selectedCacheFile: Number?;
    operation updateCache();
    operation updateImageCache();
    operation deleteSelectedCacheFile();
    operation deleteSelectedCachedImageFile();
    operation deleteImageCache();
    operation fromCacheName(name:String):String;
    operation toCacheName(name:String):String;
}


trigger on new AppModel {
    this.updateCache();
}

trigger on insert f into AppModel.cachedFiles {
    if (sizeof cachedFiles == 1) {
        selectedCacheFile = 0;
    }
}

trigger on delete f from AppModel.cachedFiles {
    var i = indexof f;
    if (i >= sizeof cachedFiles) {
        i--;
    }
    if (i >= 0) {
        selectedCacheFile = i;
    }
}

trigger on insert f into AppModel.cachedImageFiles {
    if (sizeof cachedImageFiles == 1) {
        selectedCachedImageFile = 0;
    }
}

trigger on delete f from AppModel.cachedImageFiles {
    var i = indexof f;
    if (i >= sizeof cachedImageFiles) {
        i--;
    }
    if (i >= 0) {
        selectedCachedImageFile = i;
    }
}

class Binder {
    attribute triggerValue: *;
    attribute action: function();
}

trigger on Binder.triggerValue = value {
    (this.action)();
}

operation AppModel.deleteSelectedCacheFile() {
    var name = cachedFiles[selectedCacheFile];
    name = toCacheName(name);
    <<net.java.javafx.typeImpl.F3WebStart>>.deleteCachedFile(name);
    var i = selectedCacheFile;
    delete cachedFiles[i];
    if (i == sizeof cachedFiles) {
        i = sizeof cachedFiles -1;
    }
    selectedCacheFile = i;
}

operation AppModel.deleteSelectedCachedImageFile() {
    var name = cachedImageFiles[selectedCachedImageFile];
    name = toCacheName(name);
    var i = selectedCachedImageFile;
    delete cachedImageFiles[i];
    if (i == sizeof cachedImageFiles) {
        i = sizeof cachedImageFiles -1;
    }
    selectedCachedImageFile = i;
}

operation AppModel.toCacheName(name:String) {
    name = name.replace('\\', '/');
    name = name.replaceAll(":", "%3a");
    name = name.replaceAll("/", "%2f");
    name = name.replaceAll("[.]", "%2e");
    name = name.replaceAll("[&]", "%26");
    name = name.replaceAll("[=]", "%3d");
    name = name.replaceAll("[?]", "%3f");
    return name;
}

operation AppModel.fromCacheName(name:String) {
    name = name.replace('\\', '/');
    name = name.replaceAll("%3a", ":");
    name = name.replaceAll("%2f", "/");
    name = name.replaceAll("%2e", ".");
    name = name.replaceAll("%26", "&");
    name = name.replaceAll("%3d", "=");
    name = name.replaceAll("%3f", "?");
    return name;
}

operation AppModel.updateCache() {
    delete cachedFiles;
    var cacheDir = new File("{System.getProperty("user.home")}/.javafxcache/");
    var files = cacheDir.listFiles();
    for (i in files[f|not f.name.endsWith(".rec")]) {
        var name = i.getName();
        name = fromCacheName(name);
        insert name into cachedFiles;
    }
}

operation AppModel.deleteImageCache() {
    var cacheDir = new File("{System.getProperty("user.home")}/.javafxcache/images/");
    var files = cacheDir.listFiles();
    for (i in files) {
        i.<<delete>>();
    }
}

operation AppModel.updateImageCache() {
    delete cachedImageFiles;
    var cacheDir = new File("{System.getProperty("user.home")}/.javafxcache/images/");
    var files = cacheDir.listFiles();
    for (i in files[f|not f.name.endsWith(".rec")]) {
        var name = i.getName();
        name = fromCacheName(name);
        insert name into cachedImageFiles;
    }
}


APP_MODEL:AppModel = new AppModel();

APP_MANAGER:F3AppManager.addAppListener(new F3AppListener() {

    operation applicationDownloadStart(app:F3App, resourceCount:Number) {
        do later {
            println("starting download: {app.mainClassName} {resourceCount}");
            insert AppDownload {app: app, total: resourceCount} into APP_MODEL:AppModel.appDownloads;
        }
    }

    operation applicationDownloadProgress(app:F3App, resourceCount:Number) {
        do later {
            var d = APP_MODEL:AppModel.appDownloads[ad|ad.app == app];
            println("download: {app.mainClassName} {resourceCount}");
            d.progress = resourceCount;
        }
    }

    operation applicationDownloadComplete(app:F3App) {
        do later {
            var d = APP_MODEL:AppModel.appDownloads[dl|dl.app == app][0];
            println("download complete: {app.getMainClassName()}");
            for (i in [false, true]) (dur 1000) {
                if (i) {
                    d.done = true;
                    delete APP_MODEL:AppModel.appDownloads[ad|ad == d];
                }
            }
        }
    }

    operation applicationAdded(app:F3App) {
        do later {
            println("app added: {app.mainClassName}");
            insert app into APP_MODEL:AppModel.apps;
            if (sizeof APP_MODEL:AppModel.apps == 1) {
                APP_MODEL:AppModel.selection = 0;
            }
        }
    }
    operation applicationRemoved(appl:F3App) {
        do later {
            var ad = APP_MODEL:AppModel.appDownloads[dl|dl.app == appl][0];
            ad.done = true;
            delete APP_MODEL:AppModel.appDownloads[x|x == ad];
            var d = APP_MODEL:AppModel.downloads[dl|dl.app == appl][0];
            if (d <> null and d.active and not d.cancelled) {
                var message = MessageDialog {
                    title: "Error"
                    messageType: ERROR
                    message: "Download of resource {d.url} failed"
                    visible: true
                };
                d.active = false;
                delete APP_MODEL:AppModel.downloads[dl|dl == d];
            }
            println("app removed: {appl.mainClassName}");
            delete APP_MODEL:AppModel.apps[a|a == appl];
            if (sizeof APP_MODEL:AppModel.apps == 0) {
                APP_MODEL:AppModel.selection = null;
            }
        }
    }
    
    operation downloadStart(app:F3App, urlStr:String) {
        do later {
            if (sizeof APP_MODEL:AppModel.downloads[d|d.url == urlStr] > 0) {
                return;
            }
            println("creating download {urlStr}");
            insert Download {
                app: app
                url: urlStr
                active: true
                total: 0
            } as first into APP_MODEL:AppModel.downloads;
        }
    }
    
    operation downloadEnd(app:F3App, urlStr:String, completed:Boolean) {
        do later {
            var d = APP_MODEL:AppModel.downloads[d|d.url == urlStr][0];
            d.active = false;
            if (completed) {
                if (not (urlStr in APP_MODEL:AppModel.cachedFiles)) {
                    insert urlStr into APP_MODEL:AppModel.cachedFiles;
                }
                if (d.total == 0) {
                    delete APP_MODEL:AppModel.downloads[dl|dl == d];
                }
            }
        }
    }
    
    operation downloadProgress(app:F3App, urlStr:String, amount:Number, total:Number) {
        do later {
            var d = APP_MODEL:AppModel.downloads[d|d.url == urlStr];
            if (d == null) {
                return;
            }
            if (amount > 0 and d.progress == 0) {
                APP_MODEL:AppModel.totalProgress++;
            }
            d.progress = amount;
            d.total = total;
            var dl = APP_MODEL:AppModel.appDownloads[ad|ad.app == app];
            dl.download = d;
            println("{amount} of {total} for {urlStr}");
        }
    }
});

APP_FRAME:Frame = Frame {
    var: frame
    title: "JavaFX"
    height: 300
    width: 500
    disposeOnClose: false
    var downloadWindows = bind foreach (d in APP_MODEL:AppModel.appDownloads) 
                Frame {
                  var: win
                  visible: true
                  trigger on (downLoadComplete = d.done) {
                      for (i in [false, true]) (dur 2000 linear) {
                         if (i) {
                             win.close();
                         }
                       }
                  }
                  resizable: false
                  title: "JavaFX: {d.app.getMainClassName()}"
                  centerOnScreen: true
                  disposeOnClose: true
                  content: 
                  GroupPanel {
                      background: white
                      var dl = bind d.download
                      var row1 = Row {resizable: false, alignment: LEADING}
                      var row2 = Row {resizable: false, alignment: LEADING}
                      var row3 = Row {resizable: false, alignment: LEADING}
                      var row4 = Row {resizable: false, alignment: LEADING}
                      var row5 = Row {resizable: true, alignment: TRAILING}
                      var col = Column {resizable: true}
                      function formatValue(n:Number) = 
                              if (n < 0) then "unknown size"
                              else if (n <= 1000) 
                              then "{n} bytes"
                              else if (n <= 1000000) 
                              then "{(n/1000).intValue()} KB"
                              else "{n/1000000 format as <<0.00>>} MB";
                      rows: [row1, row2, row3, row4, row5]
                      //background: white
                      columns: col
                      content:
                      [Label {opaque: true
                          alignmentY: 0
                          row: row1
                          column: col
                          text: bind 
                             "<html>
                                <table>
                                   <!-- tr>
                                     <td>
                                      <img src='file:net/java/javafx/ui/resources/f3.png'/></img>
                                     </td>
                                   </tr-->
                                   <tr>
                                       <td>
                                       <b>Starting:</b>
                                       </td>
                                       <td>
                                       {d.app.getMainClassName()}
                                       </td>
                                   </tr>
                                   <tr>
                                       <td>
                                       <b>Loaded:</b>
                                       </td>
                                       <td>
                                       {d.progress.intValue()} of {d.total.intValue()} resource(s)                                   
                                       </td>
                                   </tr>
                                </table>
                            </html>"
                          horizontal: {min: -2, max: -2}
                          //vertical: {min: -2, max: -2, pref: 60}
                      },
                      ProgressBar {opaque: true
                          var: self
                          row: row2
                          column: col
                          min: 0
                          max: bind d.total
                          value: bind d.progress
                      },
                      Label {opaque: true
                          alignmentY: 0
                          row: row3
                          column: col
                          text: bind "<html><table><tr><td nowrap><b>Resource:</b> </td><td nowrap>{dl.url}</td></tr><tr><td nowrap><b>Loaded:</b></td><td>{if dl.progress < 0 then "" else "{formatValue(dl.progress)} of {formatValue(dl.total)}"} {if dl.total <= 0 then "" else "{((dl.progress/dl.total)*100).intValue()}%"}</td></tr></table></html>"
                          horizontal: {min: -2, max: -2, pref: 350}
                          //vertical: {min: -2, max: -2, pref: 60}
                      },
                      ProgressBar {opaque: true
                          var: self
                          row: row4
                          column: col
                          min: 0
                          max: 100
                          value: bind if dl == null then 0 else if dl.total > 0 then ((dl.progress/dl.total)*100).intValue() else if dl.total == 0 and dl.progress == 0 then 100 else 0
                          indeterminate: bind dl <> null and dl.total < 0
                      },
                      FlowPanel {
                           column: col
                           row: row5
                           alignment: TRAILING
                           vgap: 0, hgap: 0
                           content: Button {
                                text: "Cancel"
                                action: operation() {
                                    win.close();
                                    d.app.destroy();
                                }
                           }
                        }]
                  }
              }

    menubar: MenuBar {
        menus: Menu {
            text: "File"
            mnemonic: F
            items: 
            [MenuItem {
                text: "Close"
                mnemonic: C
                action: operation() {
                    frame.visible = false;
                }
            },
            MenuSeparator,
            MenuItem {
                text: "Garbage Collect"
                mnemonic: G
                action: operation() {<<net.java.javafx.typeImpl.F3WebStart>>.clearClassLoaderCache(); System.gc();}
            },
            MenuItem {
                text: "Delete Cache"
                mnemonic: D
                action: operation() {
                    <<net.java.javafx.typeImpl.F3WebStart>>.deleteCache();
                    APP_MODEL:AppModel.updateCache();
                }
            },
            MenuItem {
                text: "Delete Image Cache"
                mnemonic: I
                action: operation() {
                    APP_MODEL:AppModel.deleteImageCache();
                }
            },
            MenuSeparator,
            MenuItem {
                text: bind if APP_MODEL:AppModel.profiling then "Stop Profiler" else "Start Profiler"
                mnemonic: P
                action: operation() {
                    if (not APP_MODEL:AppModel.profiling) {
                        APP_MODEL:AppModel.profiling = true;
                        do later {
                            <<net.java.javafx.typeImpl.TypeFactoryImpl>>.startProfiler();
                        }
                    } else {
                        APP_MODEL:AppModel.profiling = false;
                        <<net.java.javafx.typeImpl.TypeFactoryImpl>>.stopProfiler();
                    }
                }
            },
            MenuItem {
                text: "Dump Profile"
                mnemonic: U
                action: operation() {
                    <<net.java.javafx.typeImpl.TypeFactoryImpl>>.dumpProfile();
                }
            },
            MenuSeparator,
            MenuItem {
                text: "Shutdown"
                mnemonic: S
                action: operation() {System.exit(0);}
            }]
        }
    }
    content: TabbedPane {
        tabs:
        [/*Tab {
            title: "Downloads"
            content:
            GroupPanel {
                opaque: true
                var row1 = Row {resizable: true}
                var col1 = Column {resizable: true}
                rows: row1
                columns: col1
                content:
                BorderPanel {
                    row: row1
                    column: col1
                    bottom: FlowPanel {
                        alignment: TRAILING
                        content: Button {
                            text: "Clean Up"
                            action: operation() {
                                delete APP_MODEL:AppModel.downloads[not active];
                            }
                        }
                    }center:
                    Canvas {
                        cursor: DEFAULT
                        var: c
                        background: white
                        content:
                        View {
                            size: bind select {width: w, height: h} from w in c.viewport.currentWidth, h in c.viewport.currentHeight
                            content:
                            ScrollPane {
                                horizontalScrollBarPolicy: NEVER
                                view:
                                Canvas {
                                    cursor: DEFAULT
                                    var: canvas
                                    content:
                                    View {
                                        size: bind select {width: w} from w in canvas.viewport.currentWidth
                                        
                                        content:
                                        Box {
                                            orientation: VERTICAL
                                            content:
                                            bind foreach (d in APP_MODEL:AppModel.downloads) 
                                            BorderPanel {
                                                top:
                                                GroupPanel {
                                                    visible: bind d.progress <> 0  or d.total <> 0
                                                    var row1 = Row {resizable: true}
                                                    var row2 = Row {resizable: true}
                                                    var row3 = Row {resizable: true}
                                                    var col = Column {resizable: true}
                                                    rows: [row1, row2, row3]
                                                    background: white
                                                    columns: col
                                                    content:
                                                    [SimpleLabel {
                                                        //                          icon: Image {url: ""}
                          row: row1
                                                        column: col
                                                        text: bind "<html><table><tr> <td valign='top'><img src='file:net/java/javafx/ui/resources/f3.png' ></img></td><td halign='left'>resource: {d.url}</td></tr></table></html>"
                                                    //                          horizontal: {min: -2, max: -2}
                      },
                                                    ProgressBar {
                                                        var: self
                                                        row: row2
                                                        column: col
                                                        min: 0
                                                        max: 100
                                                        indeterminate: bind d.total < 0
                                                        function formatValue(n:Number) = 
                                                        if (n < 0) then "unknown size"
                                                        else if (n <= 1000) 
                                                        then "{n} bytes"
                                                        else if (n <= 1000000) 
                                                        then "{(n/1000).intValue()} KB"
                                                        else "{n/1000000 format as <<0.00>>} MB";
                                                        
                                                        value: bind if d.total > 0 then ((d.progress/d.total)*100).intValue() else if d.total == 0 and d.progress == 0 then 100 else 0
                                                        string: bind "{formatValue(d.progress)} of {formatValue(d.total)}        {self.value}%"
                                                        stringPainted: bind d.progress > 0
                                                    },
                                                    FlowPanel {
                                                        column: col
                                                        row: row3
                                                        alignment: TRAILING
                                                        vgap: 0
                                                        hgap: 0
                                                        content: Button {
                                                            text: bind if d.active then "Cancel" else "Remove"
                                                            action: operation() {
                                                                if (d.active) {
                                                                    var app = d.app;
                                                                    d.cancelled = true;
                                                                    d.active = false;
                                                                    app.destroy();
                                                                } else {
                                                                    delete APP_MODEL:AppModel.downloads[dl|dl==d];
                                                                }
                                                            }
                                                        }
                                                    }]
                                                } 
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            },*/
        Tab {
            title: "Applications"
            content:
            GroupPanel {
                opaque: true
                var col = Column {resizable: true, alignment: TRAILING}
                var row1 = Row {resizable: true, alignment: LEADING}
                var row2 = Row {}
                columns: col
                rows: [row1, row2]
                content: 
                [ListBox {
                    row: row1
                    column: col
                    selection: bind APP_MODEL:AppModel.selection
                    cells: bind foreach (app in APP_MODEL:AppModel.apps)
                    ListCell {
                        text: app.mainClassName
                    }
                },
                FlowPanel {
                    vgap: 0
                    hgap: 0
                    alignment: TRAILING
                    row: row2
                    column: col
                    content: Button {
                        alignmentX: 1.0
                        text: "End Process"
                        mnemonic: E
                        enabled: bind sizeof APP_MODEL:AppModel.apps > 0
                        action: operation() {
                            APP_MODEL:AppModel.apps[APP_MODEL:AppModel.selection].destroy();
                        }
                    }
                }]
            }
        },
        Tab {
            title: "Cache"
            content: 
            GroupPanel {
                opaque: true
                var col = Column {resizable: true, alignment: TRAILING}
                var row1 = Row {resizable: true, alignment: LEADING}
                var row2 = Row {}
                columns: col
                rows: [row1, row2]
                content: 
                [ListBox {
                    row: row1
                    column: col
                    selection: bind APP_MODEL:AppModel.selectedCacheFile
                    cells: bind foreach (f in APP_MODEL:AppModel.cachedFiles)
                    ListCell {
                        text: f
                    }
                },
                FlowPanel {
                    vgap: 0
                    hgap: 0
                    alignment: TRAILING
                    row: row2
                    column: col
                    content: Button {
                        alignmentX: 1.0
                        text: "Delete"
                        mnemonic: D
                        enabled: bind sizeof APP_MODEL:AppModel.cachedFiles > 0
                        action: operation() {
                            APP_MODEL:AppModel.deleteSelectedCacheFile();
                        }
                    }
                }]
            }
        }]
    }
};


